/* BUILDFLAG:"DS26528E1 = 1" */
/* 
 * Copyright 1997-2005 NComm, Inc.  All rights reserved.
 * 
 *
 *                     *** Important Notice ***
 * The software contained in this file may only be used within a 
 * valid license agreement between your company and NComm, Inc. 
 * The license agreement includes the definition of a PROJECT which 
 * defines the scope that this software can be used in.  Any use outside of
 * the definition of the PROJECT is prohibited without executing an additional 
 * agreement with NComm, Inc. Please refer to you license agreement for 
 * the definition of the PROJECT.
 *
 * Verification of your company's license agreement and copies of 
 * that agreement also may be obtained from:
 *
 * NComm, Inc.
 * 254 N Broadway
 * Suite 106
 * Salem, NH 03079
 * (603) 893-6186
 * sales@ncomm.com
 *
 */

/*
 *	The Device version is the chip revision that is supported by this
 *	driver.
 *
 *	Device Version:
 *		Rev A3
 *
 *	Revision:
 *		08/11/04 - Initial Release
 *		10/10/05 - Tested on A3 Rev Silicon
 *
 *	See CUSTOMIZE HERE for areas that may need to be customized for your
 *	platform.
 */


/*
 * Driver for the Dallas DS26528 DS26528 chip, E1 section
 */

/* -------------------------------------------------------------------------- */
/* Device Issues:  This section lists any issues with the device.
 * The list contains descriptions of:
 * 1) Lack of device features which prevent the NComm TMS software 
 *    from meeting the applicable standards.
 * 2) Work arounds employed to make the NComm TMS software package(s) operate
 *    properly.
 * 3) Special hardware considerations for implementing the device.
 *
 */

#if 0
Device Issues:

1) EU certification test with the Marquest FRAI occasionally fail the TBR4-B53
and CL44 sequences. We are working on determining the cause.

#endif

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* Driver Customization Section                                   
 * This section contains any specific customizations required for this chip
 * and/or driver operation.
 * The customizations cover the chip configuration and interfaces to
 * other parts of your platform. 
 */

	/* These get placed in the DEVICE call in the CTRL and POLL driver API.
	 * It allows for platform specific Control and Poll calls .
	 */

#define CUSTOM_CTRL(LnPtr,uparam1,uparam2,uparam3,uparam4) 0

#define CUSTOM_POLL(LnPtr,uparam1,uparam2,uparam3,uparam4) 0


/* -------------------------------------------------------------------------- */
/* Define default E1 technology constraints
 */

#define E1_DRIVER
#define MAX_E1_TIMESLOTS	32
#define TSLOT_IDLECODE		0xD5	/* default idle code */
#define SIGNL_IDLECODE		0x0B	/* default idle abcd bits for CAS */
#define DEFAULT_TX_CLOCK	SYSCLOCK

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* Include files
 */

#include "target.h"
#include "apiTE1Dv.h"
#include "ds26528.h"


/* -------------------------------------------------------------------------- */
/* Define API Entry Points
 * These are the names you will use in the API registration processor for
 * this driver
 */

#define TE1_CTRLENTRY _ds26528E1CTRL
#define TE1_POLLENTRY _ds26528E1POLL

int TE1_CTRLENTRY(void *vptr, TE1DCTRL_FC fcode, ...);

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* General houskeeping strucucture on a per-line basis
 */
enum {
	INTFLAG_UTIL 	= (1 << 1),	/* Utility interrupt */
	INTFLAG_ALM  	= (1 << 2),	/* Alarm interrupt */
	INTFLAG_ONESEC  = (1 << 3)	/* One second interrupt */
};


struct _E1stuff {
	FRAMER *myfrmr;
	DEVICE *mydevice;
	LIU *myliu;
	int mysubc;
	int rxslip;

	void *sasiThandle;
};


struct _stuff {
	int greset;			/* Tracks global reset */
	int init;			/* 0 = not initalized, 1 = initalized */
	int line;			/* Line Number */
	int subc;			/* sub channel number */
	void *lnPtr;			/* pointer to talk to TMS with */
	void *LIUptr;			/* pointer to talk to LIU with */
	GDEVICE *d;			/* reverse pointer to the dev struct */
	int (*cback)(void *LnPtr, TE1DCLBK_FC,...);
	int int_flag;			/* deterines which interrupts enabled */
	int isr_hooked;			/*  1 = ISR is hooked, 0 = not */

	int utility_init;		/* 0 = util not init, 1 initalized */
	int alarm_init;			/* 0 = alarms not inialized, 1 enabled*/
	int onesec_init;		/* 0 = disabled, 1 = enabled */

	int g826_init;			/* 0 = disabled, 1 = enabled */
	int signal_init;		/* 0 = CAS disabled, 1 = CAS enabled */
	int loop_init;			/* 0 = disabled, 1 = enabled */

	TE1_CONFIG cfg;			/* Current configuration */
	int side;			/* USR_SIDE or NET_SIDE */
	CLOCK_TYPE txclk;		/* current setting for transmit clock */

	int loopback;			/* Loopback information */

	int tx_alm;			/* current transmit alarms */

	struct {
		enum {
			DS0MODE_CLEAR,
			DS0MODE_IDLE,
			DS0MODE_RBCAS
		} mode; 		/* Mode of the DS0 */
		unsigned char ipat;	/* Idle Pattern */
		unsigned char rxbits;	/* Receive Signaling Bits */
		unsigned char txbits;	/* Transmit Signaling Bits */
	} ds0[MAX_E1_TIMESLOTS];
	
	int sasi_init;			/* 0 = disabled, 1 = enabled */
	unsigned char sa_bits;		/* Holds the Tx SaBits - nonCRC */
	unsigned char sabyte[E1SABYTE_MAX]; /* Holds the Tx SaBytes - CRC */
	unsigned char si_bits;		/* Holds the International Bits */
	unsigned char x_bits;		/* Holds the Extra Bits */

	struct _E1stuff e1s;		/* Device specific items */
};

typedef struct _stuff STUFF;

static STUFF E1stuff[MAX_DEVICES][MAX_E1_CHANNELS];

/* -------------------------------------------------------------------------- */
/* Forward References
 */
static int hook_isr(STUFF *sptr, int set_clear, int flag);
static int poll_allrxbits(STUFF *sptr, int *all_rxbits);

/* -------------------------------------------------------------------------- */
/* Define Helper Defines 
 * This is a bit map.
 */

/* Loopbacks */
#define DRV_LOOP_REMOTE 0x01
#define DRV_LOOP_LOCAL 0x02
#define DRV_LOOP_PAYLOAD 0x04
#define DRV_LOOP_FRAMER 0x08

/* Testing transmit Alarms */
#define TXA_ISSET(x) (sptr->tx_alm & (x) & (~TMS_TXALARM))

#define STANDARD_DECLARE FRAMER *f = sptr->e1s.myfrmr;

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* The Device Specific Routines are defined below.
 */

/* -------------------------------------------------------------------------- */
/* Routine to look up the appopriate stuff stucture for the line
 * The pointer returned is local to the E1 driver
 */
static STUFF *_te1_get_stuff(GDEVICE *d, int subc)
{
	return(&E1stuff[d->id][subc]);
}

/* -------------------------------------------------------------------------- */
/* This routine fills out the device specific stuff information
 */
static void _te1_set_chip_stuff(STUFF *sptr)
{
	sptr->e1s.mydevice = (DEVICE *) sptr->d->BaseAdr;
	sptr->e1s.mysubc = sptr->subc % MAX_E1_CHANNELS;
	sptr->e1s.myfrmr = (FRAMER *) 
		&sptr->e1s.mydevice->te1[sptr->e1s.mysubc];
	sptr->e1s.myliu = (LIU *) 
		&sptr->e1s.mydevice->bertliu.liu[sptr->e1s.mysubc];
	sptr->e1s.rxslip = 0;
	return;
}

#define BYTE_FLIP(x) ((((x) & 0x80) >> 7) | (((x) & 0x40) >> 5) | \
	(((x) & 0x20) >> 3) | (((x) & 0x10) >> 1) | \
	(((x) & 0x08) << 1) | (((x) & 0x04) << 3) | \
	(((x) & 0x02) << 5) | (((x) & 0x01) << 7))

#define FLIP_EM(x) ((((x) & 0x08) >> 3) | (((x) & 0x04) >> 1) | \
	(((x) & 0x02) << 1) | (((x) & 0x01) << 3))

static int flip_table[16] = {
	FLIP_EM(0),
	FLIP_EM(1),
	FLIP_EM(2),
	FLIP_EM(3),
	FLIP_EM(4),
	FLIP_EM(5),
	FLIP_EM(6),
	FLIP_EM(7),
	FLIP_EM(8),
	FLIP_EM(9),
	FLIP_EM(10),
	FLIP_EM(11),
	FLIP_EM(12),
	FLIP_EM(13),
	FLIP_EM(14),
	FLIP_EM(15),
	};


/*------------------------------------------------------------------------*/
static void process_SR1_interrupt(STUFF *sptr)
{
volatile unsigned char status, statusl;
STANDARD_DECLARE;

	status = f->rrts1;
	statusl = f->rls1;

	if(statusl & (RLS1_RRAIC | RLS1_RRAID)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXRAI, 
			       (status & RRTS1_RRAI) ? 1 : 0);
	}
	
	if(statusl & (RLS1_RAISC | RLS1_RAISD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXAIS, 
			       (status & RRTS1_RAIS) ? 1 : 0);
	}

	if(statusl & (RLS1_RLOSC | RLS1_RLOSD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXLOS, 
			       (status & RRTS1_RLOS) ? 1 : 0);
	}

	if(statusl & (RLS1_RLOFC | RLS1_RLOFD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXOOF, 
			       (status & RRTS1_RLOF) ? 1 : 0);
	}

	f->rls1 = statusl;

}


/*------------------------------------------------------------------------*/
static void process_SR4_interrupt(STUFF *sptr)
{
volatile unsigned char statusl;
STANDARD_DECLARE;

	statusl = f->rls4;
	
	if(statusl & RLS4_TIMER) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ONESEC);
	}

	if(statusl & RLS4_RSLIP) {
		sptr->e1s.rxslip = 1;
	}

	f->rls4 = statusl;
}


/*------------------------------------------------------------------------*/
/* This routine performs the utility initialization
 */
static int utility_init(STUFF *sptr, int set_clear)
{
int retval = 1;
FRAMER *f = (FRAMER *)&sptr->e1s.mydevice->te1[0];

	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_UTIL);
		f->gfimr |= (1 << sptr->e1s.mysubc);
		if(!retval)
			return(0);
	}
	else {
		f->gfimr &= ~(1 << sptr->e1s.mysubc);
		retval = hook_isr(sptr, set_clear, INTFLAG_UTIL);
		if(!retval)
			return(0);
	}
			
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine sets up for signaling 
 */
static int signal_init(STUFF *sptr, int set_clear)
{
int retval = 1;
int val[MAX_E1_TIMESLOTS];
int *sig = &val[0];

	/* The can only enable signaling on MF E1s */

	if(sptr->cfg.framing == FF_E1_CRC ||
	   sptr->cfg.framing == FF_E1)
		return(0);

	if(set_clear) {
		retval = (*sptr->cback)(sptr->lnPtr, TE1DCLBK_SIGNLDEBOUNCE, 1);
		if(!retval)
			return(0);
		poll_allrxbits(sptr, sig);
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as clear channel - no CAS
 */
static int signal_clear(STUFF *sptr, int ts, int set_clear)
{
int retval = 1;

	/* Nothing to be done */
	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as clear channel - no CAS
 */
static int signal_idle(STUFF *sptr, int ts, int ipat, int set_clear)
{
int retval = 1;
int index, bit;
STANDARD_DECLARE;

	if(ipat != -1) {
		f->tidr[ts] = ipat;
	}

	index = ts / 8;
	bit = ts % 8;

	if(set_clear) {
		f->tcice[index] |= (1 << bit);
	}
	else {
		f->tcice[index] &= ~(1 << bit);
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as needing CAS processing
 */
static int signal_rbcas(STUFF *sptr, int ts, int set_clear)
{
int retval = 1;

	/* Nothing to be done here */
	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine sets the transmit signaling bits for a CAS channel
 */
static int signal_txbits(STUFF *sptr, int ts, unsigned char dcba)
{
int retval = 1;
int index;
unsigned int bits;
STANDARD_DECLARE;

	if ((ts == 0) || (ts == 16)) return(retval);
	index = (ts > 15) ? (ts - 16) : ts;
	bits =  flip_table[dcba & 0xf];

	sptr->ds0[ts].txbits = bits;

	/* change bits = 0xDCBA to bits = 0xABCD, move them into
	 * the proper register position, and generate a mask
	 * to keep the undisturbed bits.
	 */

	bits = (ts > 15) ? (((sptr->ds0[ts - 16].txbits) << 4) | bits) : 
	       ((sptr->ds0[ts + 16].txbits) | (bits << 4));	
	f->ts[index] = bits;

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine forces a receive framer to re-acquire frame
 */
static int do_resync(STUFF *sptr)
{
int retval = 1;
STANDARD_DECLARE;

	f->rcr1 |= RCR1_RESYNC;
	f->rcr1 &= ~RCR1_RESYNC;

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine turns on and off SASI bit processing
 */
static int sasi_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;

	/* Let the upper layers do the SaSi bit debouncing and mittoting,
	 * and they must be handled via a periodic poll.
	*/

	(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASISADEBOUNCE, 1);

	(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASISIDEBOUNCE, 1);

	(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASIXDEBOUNCE, 1);

	if ((sptr->cfg.framing == FF_E1_CRC_MF) ||
		    (sptr->cfg.framing == FF_E1_CRC)) {
		f->tsacr |= TSACR_SA4 | TSACR_SA5 | TSACR_SA6 |
				TSACR_SA7 | TSACR_SA8;
	}
	else {
		f->tsacr &= ~(TSACR_SA4 | TSACR_SA5 | TSACR_SA6 |
				TSACR_SA7 | TSACR_SA8);
	}

	if(set_clear) {
		(*sptr->cback)(sptr, TE1DCLBK_HOOKOSTIC,
			SASI_POLL_TICK, &sptr->e1s.sasiThandle,
			TE1_CTRLENTRY, TE1DCTRL_OSTIC, SASI_OSTICKER);
	}
	else {
		(*sptr->cback)(sptr, TE1DCLBK_UNHOOKOSTIC,
					sptr->e1s.sasiThandle);
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine transmits the SA (National) bits in nonCRC Framing
 */
static int tx_sabits(STUFF *sptr, int sabits)
{
int retval = 1;
STANDARD_DECLARE;

	f->tsacr = (f->tsacr & 0xe0) | sabits;

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine transmits the SA (National) bits in CRC Framing
 */
static int tx_sabyte(STUFF *sptr, int sabyteN, int sabyte)
{
int retval = 1;
STANDARD_DECLARE;

	switch (sabyteN) {
	case E1SABYTE4:
		f->tsa4 = (unsigned char)BYTE_FLIP(sabyte);
		break;
	case E1SABYTE5:
		f->tsa5 = (unsigned char)BYTE_FLIP(sabyte);
		break;
	case E1SABYTE6:
		f->tsa6 = (unsigned char)BYTE_FLIP(sabyte);
		break;
	case E1SABYTE7:
		f->tsa7 = (unsigned char)BYTE_FLIP(sabyte);
		break;
	case E1SABYTE8:
		f->tsa8 = (unsigned char)BYTE_FLIP(sabyte);
		break;
	default:
		retval = 0;
		break;
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine transmits the Si (International) bits
 */
static int tx_sibits(STUFF *sptr, int sibits)
{
int retval = 1;
STANDARD_DECLARE;

	/* The Si bit position is consumed by the CRC */

	if(sptr->cfg.framing == FF_E1_CRC ||
	   sptr->cfg.framing == FF_E1_CRC_MF)
		return(0);

	f->tslc3_tsiaf = f->tsinaf = (sibits) ? 0xff : 0;

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine transmits the extra (X) bits in a MF framed E1
 */
static int tx_xbits(STUFF *sptr, int xbits)
{
int retval = 1;
int bits;
long lval;
STANDARD_DECLARE;

	/* The X bits are only available with CAS signaling */

	if(sptr->cfg.framing == FF_E1_CRC ||
	   sptr->cfg.framing == FF_E1)
		return(0);

	lval = INTS_OFF();
	bits = f->ts[0] & 0xf4;	
	bits |= ((xbits & 0x03) | ((xbits & 0x04) << 1));
	f->ts[0] = bits;
	INTS_ON(lval);

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will poll alarms on the appriate line
 */
static int poll_alarm(STUFF *sptr, int alarm_type, int *valueptr)
{
int retval = 1;
int value = 0;
STANDARD_DECLARE;

	switch(alarm_type) {
	case TMS_RXLOS:

		value = (f->rrts1 & RRTS1_RLOS) ? 1 : 0;
		break;

	case TMS_RXOOF:

		value = (f->rrts1 & RRTS1_RLOF) ? 1 : 0;
		break;

	case TMS_RXAIS:
		value = (f->rrts1 & RRTS1_RAIS) ? 1 : 0;
		break;

	case TMS_RXRAI:
		value = (f->rrts1 & RRTS1_RRAI) ? 1 : 0;
		break;

	default:
		retval = 0;
		break;
	}

	*valueptr = value;
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine will retreive the loop information
 */
static int poll_loop(STUFF *sptr, int loop_type, int *set_clear)
{
int retval = 1;
int value = 0;
LIU *l = sptr->e1s.myliu;
STANDARD_DECLARE;

	switch(loop_type) {
	case LOOP_REMSWITCH:
		/* Remote is when the the receive signal is looped back
		 * at the analog layer to the remote device.
		 * This is also known as line loopback
		 */  

		value = (f->rcr3 & RCR3_RLB) ? 1 : 0;

		break;

	case LOOP_LOCSWITCH:
		/* Local Loopback is when the voice channels are looped
		 * before the framer section.
		 */

		value = (l->lmcr & LMCR_LLB) ? 1 : 0;

		break;

	case LOOP_PAYSWITCH:
		/* Payload Loopback is when the framer receives the signal
		 * then, after the framer, the voice channels are re-sent out
		 * and the overhead is re-generated.
		 */

		value = (f->rcr3 & RCR3_PLB) ? 1 : 0;

		break;

	case LOOP_FRMSWITCH:	
		/* Framer Loopback is when the signal is set out of
		 * the framer and looped back to the framer but
		 * before it hits the LIU.  You will send and AIS while
		 * this happens.  The AIS is done by the chip and not the
		 * Upper layers.
		 */

		value = (f->rcr3 & RCR3_FLB) ? 1 : 0;

		break;

	default:
		retval = 0;
		break;
	}

	*set_clear = value;
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will retreive ALL 30 channels of signaling
 * It will fill the array of all_rxbits with all signaling channels
 * all_rxbits must be 32 ints long, but index 0 and 16 are not used
 * because they carry the framing channel and the CAS channel respecitively
 */
static int poll_allrxbits(STUFF *sptr, int *all_rxbits)
{
int retval = 1;
int val, all_bits, ts;
int *p;
STANDARD_DECLARE;

	/* note: Skip 0 which is TS 0 and TS 16 */
	p = all_rxbits;

	for (ts = 1; ts < MAX_E1_TIMESLOTS / 2; ts++) {
		all_bits = f->rs[ts];  /* this gets two time slots */

		val = flip_table[((all_bits & 0xf0) >> 4)];
		*(p + ts) = sptr->ds0[ts].rxbits = val;


		/* process the 2nd timeslot */
		val = flip_table[(all_bits & 0x0f)];
		*(p + ts + 15) = sptr->ds0[ts].rxbits = val;
		
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine will retreive the performance data
 */
static int get_pm(STUFF *sptr, E1_PERFINFO *pm)
{
int retval = 1;
STANDARD_DECLARE;

	pm->CVcount = (f->lcvcr1 << 8) | f->lcvcr2;
	pm->CRCcount = (f->pcvcr1 << 8) | f->pcvcr2;
	pm->EBITcount = (f->ebcr1 << 8) | f->ebcr2;
	pm->FAScount = (f->foscr1 << 8) | f->foscr2;

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This will retreive the SA (National) bits on the line in nonCRC Framing
 */
static int poll_sabits(STUFF *sptr, int *sabits)
{
int retval = 1;
STANDARD_DECLARE;

	*sabits = f->rslc2_rnaf & 0x1f;

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will retreive the SA (National) bits on the line in CRC Framing
 */
static int poll_sabyte(STUFF *sptr, int sabyteN, int *sabyte)
{
int retval = 1;
STANDARD_DECLARE;
	
	switch (sabyteN) {
	case E1SABYTE4:
		*sabyte = (int)BYTE_FLIP(f->rsa4);	
		break;
	case E1SABYTE5:
		*sabyte = (int)BYTE_FLIP(f->rsa5);	
		break;
	case E1SABYTE6:
		*sabyte = (int)BYTE_FLIP(f->rsa6);	
		break;
	case E1SABYTE7:
		*sabyte = (int)BYTE_FLIP(f->rsa7);	
		break;
	case E1SABYTE8:
		*sabyte = (int)BYTE_FLIP(f->rsa8);	
		break;
	default:
		retval = 0;
		break;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will retreive the Si (International) bits on the line
 */
static int poll_sibits(STUFF *sptr, int *sibits)
{
int retval = 1;
STANDARD_DECLARE;

	/* The Si bit position is consumed by the CRC */

	if(sptr->cfg.framing == FF_E1_CRC ||
	   sptr->cfg.framing == FF_E1_CRC_MF)
		return(0);

	*sibits = (f->rslc2_rnaf & 0x80) ? 1 : 0;

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will retreive the X (Extra) bits on the line
 */
static int poll_xbits(STUFF *sptr, int *xbits)
{
int retval = 1;
int bits;
STANDARD_DECLARE;

	/* The X bits are only available with CAS signaling */

	if(sptr->cfg.framing == FF_E1_CRC ||
	   sptr->cfg.framing == FF_E1)
		return(0);

	bits = f->rs[0];
	*xbits = ((bits & 0x03) | (bits & 0x08) >> 1);

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will transmit on the appriate line 
 */
static int transmit_alarm(STUFF *sptr, int alarm_type, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;

	switch(alarm_type) {
	case TMS_TXRAI:
		if(set_clear) {
			f->tsacr |= TSACR_RA;
		}
		else {
			f->tra = 0;
			f->tsacr &= ~TSACR_RA;
		}
		break;
	case TMS_TXAIS:
		if(set_clear) {
			f->tcr1 |= TCR1_TAIS;
		}
		else {
			f->tcr1 &= ~TCR1_TAIS;
		}
		break;

	default:
		retval = 0;
		break;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable G.826 processing on the line 
 */
static int g826_init(STUFF *sptr, int set_clear)
{
int retval = 1;

	/* Nothing to de done here */
	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable one sec processing on the approriate line
 */
static int onesec_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;


	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_ONESEC);
		if(!retval)
			return(0);
		f->rim4 |= RIM4_TIMER;
	}
	else {
		f->rim4 &= ~RIM4_TIMER;
		retval = hook_isr(sptr, set_clear, INTFLAG_ONESEC);
		if(!retval)
			return(0);
	}
	return(retval);
}

#if (MAX_GLOBAL_TICKERS	> 0)

/* -------------------------------------------------------------------------- */
/* The device has global tickers associated with it so,
 * the following routine is called by the global file for processing of
 * the global tickers
 */
static void E1GlobalTickerHandler(GDEVICE *d, int ticker)
{

	switch(ticker) {
	case 0:
		break;
	default:
		break;
	}
}


/*------------------------------------------------------------------------*/
/* This routine is called by global when the first time a ticker is
 * registered.  This routine makes the call back to the approprate TMS
 * to get the ticker hooked up.
 */
static void E1RegTicker(STUFF *sptr, void (*global_handler)(),
	int on_off, int ticker, int ms, void *handle)
{

	if(on_off)
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_HOOKOSTIC, ms, handle,
			global_handler, sptr->d, ticker);
		
	else
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_UNHOOKOSTIC, 
			*((unsigned long *)(handle)));
}

/*------------------------------------------------------------------------*/
/* This routine start a global ticker going
 */
static void start_global_ticker(STUFF *sptr, int ticker, int onoff)
{

	NCID_GHOOKTICKER(sptr->d, DRVR_IDX_E1, ticker, onoff,
		E1RegTicker, sptr, E1GlobalTickerHandler);
}

#endif	/* MAX_GLOBAL_TICKERS */

/*------------------------------------------------------------------------*/
/* This routine enables alarms 
 * Alarms:
 *         LOS OOF AIS RAI
 * If  MF, then Loss of MF alarm
 * If CRC, then Loss of CRC-4 framing
 */
static int alarm_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;

	/* Tell the app that the driver is doing Auto-RAI.  Off by default.
	 */
	(*sptr->cback)(sptr->lnPtr, TE1DCLBK_AUTORAI, 1);

	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_ALM);
		if(!retval)
			return(0);

		f->rim1 |= RIM1_RRAIC | RIM1_RAISC | RIM1_RLOSC |
			RIM1_RLOFC | RIM1_RRAID | RIM1_RAISD |
			RIM1_RLOSD | RIM1_RLOFD;
		f->tcr2 |= TCR2_ARA;
	}
	else {
		f->rim1 |= ~(RIM1_RRAIC | RIM1_RAISC | RIM1_RLOSC |
			RIM1_RLOFC | RIM1_RRAID | RIM1_RAISD |
			RIM1_RLOSD | RIM1_RLOFD);

		retval = hook_isr(sptr, set_clear, INTFLAG_ALM);
		if(!retval)
			return(0);
	}
			
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine sets up loopbacks of the indicated type
 */

static int do_loopback(STUFF *sptr, int looptype, int set_clear,
	int bits, int code)
{
int retval = 1;
LIU *l = sptr->e1s.myliu;
STANDARD_DECLARE;

	switch(looptype) {
	case LOOP_REMSWITCH:
		/* Remote is when the the receive signal is looped back
		 * at the analog layer to the remote device.
		 * This is also known as line loopback
		 */

		if(set_clear) {
			f->rcr3 |= RCR3_RLB;
		}
		else {
			f->rcr3 &= ~RCR3_RLB;
		}

		break;

	case LOOP_LOCSWITCH:
		/* Local Loopback is when the voice channels are looped
		 * before the framer section.
		 */
		if(set_clear) {
			l->lmcr |= LMCR_LLB;
		}
		else {
			l->lmcr &= ~LMCR_LLB;
		}

		break;

	case LOOP_PAYSWITCH:
		/* Payload Loopback is when the framer receives the signal
		 * then, after the framer, the voice channels are re-sent out
		 * and the overhead is re-generated.
		 */
		if(set_clear) {
			f->rcr3 |= RCR3_PLB;
		}
		else {
			f->rcr3 &= ~RCR3_PLB;
		}
		break;

	case LOOP_FRMSWITCH:	
		/* Framer Loopback is when the signal is set out of
		 * the framer and looped back to the framer but
		 * before it hits the LIU.  You will send and AIS while
		 * this happens.  The AIS is done by the chip and not the
		 * Upper layers.
		 */


		if(set_clear) {
			f->rcr3 |= RCR3_FLB;
		}
		else {
			f->rcr3 &= ~RCR3_FLB;
		}
		break;

	default:
		retval = 0;
		break;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine turns on and off loop detection 
 */
static int loop_init(STUFF *sptr, int set_clear)
{
int retval = 1;

	/* Nothing to do here */
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine sets up the clock used for the transmitter 
 */
static int set_txclock(STUFF *sptr, CLOCK_TYPE txclk)
{
int retval = 1;
STANDARD_DECLARE;

	switch(txclk) {
	case SYSCLOCK:	/* clock from central timing source */
		f->tcr3 = (f->tcr3 & (~TCR3_TCSS(3))) | TCR3_TCSS(0);
		break;

	case RXCLOCK:	/* loop timing selection */
		f->tcr3 = (f->tcr3 & (~TCR3_TCSS(3))) | TCR3_TCSS(3);
		break;

	case LOCALOSC:	/* clock from a local oscillator */
	case OTHERCLK:	/* some other clock selection */
	default:
		retval = 0;
		break;
	}
	
	return(retval);
}

/*------------------------------------------------------------------------*/
/* 
 * This routine sets up the configration of the channel
 * sptr is the pointer to the stuff structure
 */
static int _e1Config(STUFF *sptr)
{
int retval = 1;
STANDARD_DECLARE;

	switch(sptr->cfg.framing) {
	case FF_E1:
		/* Transmit */
		f->tcr1 &= ~(TCR1_TCRC4); 	/* no CRC4 */
		f->tcr2 &= ~(TCR2_AEBE);	/* turn off E bits */
		/* Receive */
		f->rcr1 &= ~(RCR1_RCRC4);	/* no CRC4 */
		f->rcr1 |= RCR1_RSIGM;		/* CCS mode */
		f->tsacr |= (TSACR_SIAF | TSACR_SINAF); /* si bit control on */
		f->ssie[0] = f->ssie[1] = f->ssie[2] = f->ssie[3] = 0;  
		break;

	case FF_E1_CRC:
		/* Transmit */
		f->tcr1 |= (TCR1_TCRC4); 	/* CRC4 */
		f->tcr2 |= (TCR2_AEBE);		/* turn on E bits */
		/* Receive */
		f->rcr1 |= (RCR1_RCRC4);	/* CRC4 */
		f->rcr1 |= RCR1_RSIGM;		/* CCS mode */
		f->tsacr &= ~(TSACR_SIAF | TSACR_SINAF); /* si bit control off*/
		f->ssie[0] = f->ssie[1] = f->ssie[2] = f->ssie[3] = 0;  
		break;

	case FF_E1_MF:
		/* Transmit */
		f->tcr1 &= ~(TCR1_TCRC4); 	/* no CRC4 */
		f->tcr2 &= ~(TCR2_AEBE);	/* turn off E bits */
		/* Receive */
		f->rcr1 &= ~(RCR1_RCRC4);	/* no CRC4 */
		f->rcr1 &= ~RCR1_RSIGM;		/* CAS mode */
		f->tsacr |= (TSACR_SIAF | TSACR_SINAF); /* si bit control on */
		f->ssie[0] = f->ssie[1] = f->ssie[2] = f->ssie[3] = 0xff;  
		break;

	case FF_E1_CRC_MF:
		/* Transmit */
		f->tcr1 |= (TCR1_TCRC4); 	/* CRC4 */
		f->tcr2 |= (TCR2_AEBE);		/* turn on E bits */
		/* Receive */
		f->rcr1 |= (RCR1_RCRC4);	/* CRC4 */
		f->rcr1 &= ~RCR1_RSIGM;		/* CAS mode */
		f->tsacr &= ~(TSACR_SIAF | TSACR_SINAF); /* si bit control off*/
		f->ssie[0] = f->ssie[1] = f->ssie[2] = f->ssie[3] = 0xff;  
		break;

	default:
		return(0);
	}

	switch(sptr->cfg.coding) {
	case LC_HDB3:
		f->rcr1 |= (RCR1_RHDB3);
		f->tcr1 |= (TCR1_THDB3);
		f->ercnt |= ERCNT_LCVCRF;
		break;

	case LC_AMI:
		f->rcr1 &= ~(RCR1_RHDB3);
		f->tcr1 &= ~(TCR1_THDB3);
		f->ercnt &= ~ERCNT_LCVCRF;
		break;

	default:
		return(0);
	}
	
	return(retval);
}

/*------------------------------------------------------------------------*/
/* 
 * This routine resets the E1 portion of the chip.
 * sptr is the pointer to the stuff structure
 */
static int _te1Reset(STUFF *sptr)
{
int retval = 1, i;
LIU *l = sptr->e1s.myliu;
STANDARD_DECLARE;

	if(!sptr->greset){
		NCID_GRESET(sptr->d);
		sptr->greset = 1;
	}

	/* disable all interrupts
	 */

	utility_init(sptr, (sptr->utility_init = 0));
	alarm_init(sptr, (sptr->alarm_init = 0));
	onesec_init(sptr, (sptr->onesec_init = 0));

	/* Soft Reset into E1 mode */

	f->tmmr = TMMR_SFTRST;
	f->tmmr = TMMR_T1E1;
	
	f->rmmr = RMMR_SFTRST;
	f->rmmr = RMMR_T1E1;

	f->tmmr |= TMMR_FRM_EN | TMMR_T1E1;
	f->rmmr |= RMMR_FRM_EN | RMMR_T1E1;
	
	f->rcr1 = RCR1_RSIGM | RCR1_FRC;

	f->rcr2 = 0;

	f->ribcc_rcr2 = RCR2_RSAS(0x1f);

	f->rcr3 = (RX_UNIPOLAR) ? RCR3_IDF : 0;

	f->riocr = ((RSYNC_MODE1) ? RIOCR_RSMS1 : 0) |
		((RSYNC_MODE2_E1) ? RIOCR_RSMS2 : 0) |
		((RSYNC_IO_SELECT) ? RIOCR_RSIO : 0) |
		((RSYNC_SKIP) ? RIOCR_RSMS : 0) |
		((RSYNC_CLK_MODE_E1) ? RIOCR_RSCLKM : 0) |
		((H100_MODE) ? RIOCR_H100EN : 0) |
		((RSYNC_INV) ? RIOCR_RSYNCINV : 0) |
		((RCLK_INV) ? RIOCR_RCLKINV : 0);

	f->rescr = ((RX_ELASTIC_STORE_EN) ? RESCR_RESE : 0) |
		((RES_MIN_DELAY) ? RESCR_RESMDM : 0) |
		((RXS_ZONE_SEL) ? RESCR_RSZS : 0) |
		((GAP_CLOCK_EN) ? RESCR_RGCLKEN : 0) |
		((RX_CHAN_FORMAT) ? RESCR_RDATFMT : 0);

	f->ercnt =
		((MOS_COUNT_SELECT) ? ERCNT_MOSCRF : 0) |
		((ERROR_ACU_MODE) ? ERCNT_EAMS : 0) |
		((UPDATE_SELECT_E1) ? ERCNT_ECUS : 0) |
		((MANUAL_UPDATE_SEL) ? ERCNT_MCUS : 0) |
		((ONE_SECOND_SEL) ? ERCNT_1SECS : 0);

	f->riboc = (RIBOC_DA(DEVICE_ASSIGNMENT)) |
		((IBO_EN) ? RIBOC_IBOEN : 0) |
		((IBO_SEL) ? RIBOC_IBOSEL : 0) |
		(RIBOC_IBS(IBO_SIZE));
	
	f->tiboc = (TIBOC_DA(DEVICE_ASSIGNMENT)) |
		((IBO_EN) ? TIBOC_IBOEN : 0) |
		((IBO_SEL) ? TIBOC_IBOSEL : 0) |
		(TIBOC_IBS(IBO_SIZE));

	f->tcr1 = TCR1_TSIS;

	f->tslc1_taf = 0x1b; /* must be 1b */

	f->tslc2_tnaf = 0x40; /* must be 40 */
	
	f->tcr3 = (TX_UNIPOLAR) ? TCR3_ODF : 0;

	f->tiocr = ((TSYNC_MODE) ? TIOCR_TSM : 0) |
		((TSYNC_DOUBLE_WIDE) ? TIOCR_TSDW : 0) |
		((TSYNC_IO) ? TIOCR_TSIO : 0) |
		((TSSYNC_MODE) ? TIOCR_TSSM : 0) |
		((TSYSCLK_MODE_E1) ? TIOCR_TSCLKM : 0) |
		((TSYNCIO_INV) ? TIOCR_TSSYNCINV : 0) |
		((TSYNC_INV) ? TIOCR_TSYNCINV : 0) |
		((TCLK_INV) ? TIOCR_TCLKINV : 0);

	f->tescr = ((TX_ELASTIC_STORE_EN) ? TESCR_TESE : 0) |
		((TES_MIN_DELAY) ? TESCR_TESMDM : 0) |
		((TXS_ZONE_SEL) ? TESCR_TSZS : 0) |
		((TDATA_EN) ? TESCR_TDATAEN : 0) |
		((TGAP_CLOCK_EN) ? TESCR_TGCLKEN : 0) |
		((TX_CHAN_FORMAT) ? TESCR_TDATFMT : 0);

	/* set up for multiformat signaling */
	for (i = 1; i < MAX_E1_TIMESLOTS; i++) {
		if (signal_txbits(sptr, i, SIGNL_IDLECODE))
			sptr->ds0[i].txbits = SIGNL_IDLECODE;
	}
	f->ts[0] = 0x0B;

	l->ltrcr = ((JAT_DEPTH) ? LTRCR_JADS : 0) |
		(LTRCR_JAPS(JAT_POSITION)) |
		((LOS_CRITERIA) ? LTRCR_LSC : 0);

	l->ltitsr = ((TXTERM_OFF) ? LTITSR_TIMPTOFF : 0) |
			(LTITSR_TIMPL((TX_IMPEDANCE ? 3 : 0))) |
			LTITSR_TS(TX_IMPEDANCE);
	
	l->lrismr = ((RXTERM_OFF) ? LRISMR_RIMPOFF : 0) |
			(LRISMR_RSMS((RX_IMPEDANCE ? 3 : 0)));


	retval = _e1Config(sptr);
	if(!retval)
		return(0);

	retval = transmit_alarm(sptr, TMS_TXAIS, 1);

	/* after the configuration turn on the framer */
	f->tmmr |= TMMR_INIT_DONE;
	f->rmmr |= RMMR_INIT_DONE;

	l->lmcr = LMCR_TE;

	return(retval);
}


/*------------------------------------------------------------------------*/
/* 
 * This routine UN resets the E1 portion of the chip.
 * sptr is the pointer to the stuff structure
 */
static int _te1UNReset(STUFF *sptr)
{
int retval = 1;

	/* disable all interrupts
	 */

	utility_init(sptr, (sptr->utility_init = 0));
	alarm_init(sptr, (sptr->alarm_init = 0));
	onesec_init(sptr, (sptr->onesec_init = 0));


	retval = _te1Reset(sptr);
	if(!retval)
		return(0);

	retval = transmit_alarm(sptr, TMS_TXAIS, 1);

	if(sptr->greset) {
		NCID_GUNRESET(sptr->d);
		sptr->greset = 0;
	}

	return(retval);
}

/* -------------------------------------------------------------------------- */
/* Set the USR_SIDE vs NET_SIDE indication.
 */
 
static int set_side(STUFF *sptr, int side)
{
int retval = 1;

	/* Nothing to be done here */
	return(retval);
}

/* -------------------------------------------------------------------------- */
/* Process any polling operations here 
 */
static int process_OSTick(STUFF *sptr, int ticker)
{
int retval = 1;
int bits;
STANDARD_DECLARE;

	switch(ticker) {

	/* do Sa, Si, and Xbits
	 */
	case SASI_OSTICKER:
		/* the Si Bits are only valid in non-CRC formats
		 */
		if ((sptr->cfg.framing == FF_E1) ||
		    (sptr->cfg.framing == FF_E1_MF))
			(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASISIBIT,
				(f->rslc2_rnaf & 0x80) ? 1 : 0);

		if ((sptr->cfg.framing == FF_E1_CRC_MF) ||
		    (sptr->cfg.framing == FF_E1_CRC)) {

			int sabyte[E1SABYTE_MAX];

			sabyte[E1SABYTE4] = (int)BYTE_FLIP(f->rsa4);
			sabyte[E1SABYTE5] = (int)BYTE_FLIP(f->rsa5);
			sabyte[E1SABYTE6] = (int)BYTE_FLIP(f->rsa6);
			sabyte[E1SABYTE7] = (int)BYTE_FLIP(f->rsa7);
			sabyte[E1SABYTE8] = (int)BYTE_FLIP(f->rsa8);

			(*sptr->cback)(sptr->lnPtr, 
			      TE1DCLBK_E1SASISABYTE, sabyte);
		}
		else {
			(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASISABIT, 
					(f->rslc2_rnaf & 0x1f));
		}			
		
		/* the X bits are only valid in MF formats */
		if ((sptr->cfg.framing == FF_E1_CRC_MF) ||
		    (sptr->cfg.framing == FF_E1_MF)) {
		
			bits = f->rs[0];
			bits = (bits & 0x03) | ((bits & 0x08) >> 1);
			(*sptr->cback)(sptr->lnPtr, TE1DCLBK_E1SASIXBIT, bits);
		}
		break;
	default:
		retval = 0;
		break;

	}
	return(retval);
}

/* -------------------------------------------------------------------------- */
/* This is where the real E1 channel processing for an interrupt is done
 */
static int process_e1(STUFF *sptr)
{
int check_again = 0;
unsigned int riir;
STANDARD_DECLARE;

	riir = (f->riir & 0x09);

	if(riir & 0x01) {
		process_SR1_interrupt(sptr);
		check_again = 1;
	}

	if(riir & 0x08) {
		process_SR4_interrupt(sptr);
		check_again = 1;
	}


	return(check_again);
}

/* -------------------------------------------------------------------------- */
/* This is the interrupt service routine for the E1 functions 
 * it gets called by the global routine when the E1 interrupt is hooked up 
 * d points to the global device record.
 */
static int E1_handler(GDEVICE *d)
{
int check_again = 0;
int i;
int id;
STUFF *sptr;
unsigned char fisr, fmask;

	id = d->id;
	fisr = ((DEVICE *) d->BaseAdr)->te1[0].gfisr;
	fmask = ((DEVICE *) d->BaseAdr)->te1[0].gfimr;
	fisr = fisr & fmask;

	for (i = 0; i < MAX_E1_CHANNELS; i++) {
		if(!((sptr = &E1stuff[id][i])->init))
			continue;
		if(fisr & (1 << i))
			check_again |= process_e1(sptr);
	}

	return(check_again);
}

/* -------------------------------------------------------------------------- */
/* This routine get called from the global function to hook up the isr 
 * If the E1 the first to require ISR services, this will be called 
 * if the ISR was hooked up by another technology, this will not get called
 */
static void Glob2E1hookISR(STUFF *sptr, void *device_isr,
	int set_clear)
{

	if(set_clear) {
		/* Hook up the interrupt */
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_HOOKINTERRUPT, device_isr, 
			USER_DEFINED_HOOKINT_VALUE);
	}
	else {
		/* Unhook the interrupt */
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_UNHOOKINTERRUPT);
	}
}

/*------------------------------------------------------------------------*/
/* Routine to control hooking up and unhooking interrupts
 */
static int hook_isr(STUFF *sptr, int set_clear, int flag)
{
int retval = 1;
int previous = sptr->int_flag;

	if(set_clear) {
		/* check to see if need to hook interrupt */

		sptr->int_flag |= flag;

		switch(flag) {
		case INTFLAG_UTIL:
		default:
			break;
		case INTFLAG_ONESEC:
		case INTFLAG_ALM:
			/* Hook the interrupt here */

			if(!sptr->isr_hooked) {
			   sptr->isr_hooked = 1;
			   NCID_GHOOKISR(sptr->d, DRVR_IDX_E1,
				set_clear, Glob2E1hookISR, sptr, E1_handler);
			}
			break;
		}
	}
	else {
		/* check to see if need to unhook interrupt */

		previous &= ~flag;

		switch(flag) {
		case INTFLAG_UTIL:
		default:
			break;
		case INTFLAG_ONESEC:
		case INTFLAG_ALM:
			/* UnHook the interrupt here */

			if(
				(!(previous & INTFLAG_UTIL)) && 
				(!(previous & INTFLAG_ONESEC)) && 
				(!(previous & INTFLAG_ALM)) && 
				sptr->isr_hooked
			) {
			   NCID_GHOOKISR(sptr->d, DRVR_IDX_E1,
				set_clear, Glob2E1hookISR, sptr, E1_handler);
			   sptr->isr_hooked = 0;
		
			}
			break;
		}
		sptr->int_flag &= ~flag;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Validate the line before being bound to a device
 */
static int _te1_valid_channel(GDEVICE *d, int subchannel)
{
	/* Single channl device needs no validation */
	return(1);
}


/* -------------------------------------------------------------------------- */
/* 
 * The include that follows below incorporate the standard E1
 * driver construct.  This Construct is common accross all E1
 * drivers.  You do not want to change the contents of the file that
 * is included below since it will change ALL E1 drivers.
 */
/* -------------------------------------------------------------------------- */

#include "apiTE1.inc"
