/* BUILDFLAG:"DS26528T1 = 1" */
/* 
 * Copyright 1997-2005 NComm, Inc.  All rights reserved.
 * 
 *
 *                     *** Important Notice ***
 * The software contained in this file may only be used within a 
 * valid license agreement between your company and NComm, Inc. 
 * The license agreement includes the definition of a PROJECT which 
 * defines the scope that this software can be used in.  Any use outside of
 * the definition of the PROJECT is prohibited without executing an additional 
 * agreement with NComm, Inc. Please refer to you license agreement for 
 * the definition of the PROJECT.
 *
 * Verification of your company's license agreement and copies of 
 * that agreement also may be obtained from:
 *
 * NComm, Inc.
 * 254 N Broadway
 * Suite 106
 * Salem, NH 03079
 * (603) 893-6186
 * sales@ncomm.com
 *
 */

/*
 *	The Device version is the chip revision that is supported by this
 *	driver.
 *
 *	Device Version:
 *		Rev A3
 *
 *	Revision:
 *		08/11/04 - Initial Release
 *		10/10/05 - Tested on A3 Rev Silicon
 *
 *	See CUSTOMIZE HERE for areas that may need to be customized for your
 *	platform.
 */


/*
 * Driver for the Dallas DS26528 DS26528 chip, T1 section
 */

/* -------------------------------------------------------------------------- */
/* Device Issues:  This section lists any issues with the device.
 * The list contains descriptions of:
 * 1) Lack of device features which prevent the NComm TMS software 
 *    from meeting the applicable standards.
 * 2) Work arounds employed to make the NComm TMS software package(s) operate
 *    properly.
 * 3) Special hardware considerations for implementing the device.
 *
 */

#if 0
Device Issues:

1) This driver does not support TR-08 yet.

2) Tri-Level signalling is not supported by the device.

#endif

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* Driver Customization Section                                   
 * This section contains any specific customizations required for this chip
 * and/or driver operation.
 * The customizations cover the chip configuration and interfaces to
 * other parts of your platform. 
 */

	/* These get placed in the DEVICE call in the CTRL and POLL driver API.
	 * It allows for platform speific Control and Poll calls .
	 */

#define CUSTOM_CTRL(LnPtr,uparam1,uparam2,uparam3,uparam4) 0

#define CUSTOM_POLL(LnPtr,uparam1,uparam2,uparam3,uparam4) 0

/* -------------------------------------------------------------------------- */
/* Define default T1 technology constraints
 */

#define T1_DRIVER
#define MAX_T1_TIMESLOTS	24
#define TSLOT_IDLECODE		0x17	/* default idle code */
#define SIGNL_IDLECODE		0x0F	/* default idle abcd bits for RBS */
#define MAX_PACKET		256
#define DEFAULT_TX_CLOCK	SYSCLOCK
#define FIFOSIZE		128

/* -------------------------------------------------------------------------- */
/* This section has T1 driver specific customizations 
 */


/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* Include files
 */

#include "target.h"
#include "apiTE1Dv.h"
#include "ds26528.h"


/* -------------------------------------------------------------------------- */
/* Define API Entry Points
 * These are the names you will use in the API registration processor for
 * this driver
 */

#define TE1_CTRLENTRY _ds26528T1CTRL
#define TE1_POLLENTRY _ds26528T1POLL

int TE1_CTRLENTRY(void *vptr, TE1DCTRL_FC fcode, ...);

/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
/* General houskeeping strucucture on a per-line basis
 */
enum {
	INTFLAG_UTIL 	= (1 << 1),	/* Utility interrupt */
	INTFLAG_ALM  	= (1 << 2),	/* Alarm interrupt */
	INTFLAG_ONESEC  = (1 << 3),	/* One second interrupt */
	INTFLAG_T1403	= (1 << 4),	/* BOC and HDLC interrupt */
	INTFLAG_LOOP	= (1 << 5),	/* Loop up/loop down interrupt */
	INTFLAG_TR008	= (1 << 6)	/* TR08 interrupt */
};


struct _T1stuff {
	FRAMER *myfrmr;
	DEVICE *mydevice;
	LIU *myliu;
	int mysubc;

	int rxslip;
	unsigned char rx_hdlc_setting;
	int sendingBOC;
	int sendingPKT;
	int txremaining;

};


struct _stuff {
	int greset;			/* Tracks global reset */
	int init;			/* 0 = not initalized, 1 = initalized */
	int line;			/* Line Number */
	int subc;			/* sub channel number */
	void *lnPtr;			/* pointer to talk to TMS with */
	void *LIUptr;			/* pointer to talk to LIU with */
	GDEVICE *d;			/* reverse pointer to the dev struct */
	int (*cback)(void *LnPtr, TE1DCLBK_FC,...);
	int isr_hooked;			/*  1 = ISR is hooked, 0 = not */
	int int_flag;			/* deterines which interrupts enabled */

	int utility_init;		/* 0 = util not init, 1 initalized */
	int alarm_init;			/* 0 = alarms not inialized, 1 enabled*/
	int onesec_init;		/* 0 = disabled, 1 = enabled */
	int t1403_init;			/* 0 = disabled, 1 = enabled */
	int loop_init;			/* 0 = disabled, 1 = enabled */

	int signal_init;		/* 0 = RBS disabled, 1 = RBS enabled */
	int t1231_init;			/* 0 = disabled, 1 = enabled */
	int tr008_init;			/* 0 = disabled, 1 = enabled */
	int tr54016_init;		/* 0 = disabled, 1 = enabled */
	int cmas_init;			/* CMAS channels */
	int exz_init;			/* 0 = disabled, 1 = enabled */

	TE1_CONFIG cfg;			/* Current configuration */
	int side;			/* USR_SIDE or NET_SIDE */
	CLOCK_TYPE txclk;		/* current setting for transmit clock */

	int loopback;			/* Loopback information */

	int tx_alm;			/* current transmit alarms */

	struct {
		enum {
			DS0MODE_CLEAR,
			DS0MODE_IDLE,
			DS0MODE_RBCAS
		} mode; 		/* Mode of the DS0 */
		unsigned char ipat;	/* Idle Pattern */
		unsigned char rxbits;	/* Receive Signaling Bits */
		unsigned char txbits;	/* Transmit Signaling Bits */
	} ds0[MAX_T1_TIMESLOTS];
	
	struct fdl_pkt {
		int byte_count;
		unsigned char pkt[MAX_PACKET];
		int active;
	} pktrx,pkttx;			/* tx and rx FDL packets */

	struct fdl_boc {
		unsigned char boc;
		int active;
	} bocrx,boctx;			/* tx and rx BOCs */

	struct lcode {
		int bits;
		int code;
	} rxlup, rxldn, txlup, txldn;

	struct _T1stuff t1s;		/* Device specific items */
};

typedef struct _stuff STUFF;

static STUFF T1stuff[MAX_DEVICES][MAX_T1_CHANNELS];

/* -------------------------------------------------------------------------- */
/* Forward References
 */
static int hook_isr(STUFF *sptr, int set_clear, int flag);
static void hdlc_xmit(STUFF *sptr);

/* -------------------------------------------------------------------------- */
/* Define Helper Defines 
 * This is a bit map.
 */

/* Loopbacks */
#define DRV_LOOP_REMOTE 0x01
#define DRV_LOOP_LOCAL 0x02
#define DRV_LOOP_PAYLOAD 0x04
#define DRV_LOOP_FRAMER 0x08

/* Testing transmit Alarms */
#define TXA_ISSET(x) (sptr->tx_alm & (x) & (~TMS_TXALARM))

#define STANDARD_DECLARE FRAMER *f = sptr->t1s.myfrmr

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* The Device Specific Routines are defined below.
 */

/* -------------------------------------------------------------------------- */
/* Routine to look up the appopriate stuff stucture for the line
 * The pointer returned is local to the T1 driver
 */
static STUFF *_te1_get_stuff(GDEVICE *d, int subc)
{
	return(&T1stuff[d->id][subc]);
}

/* -------------------------------------------------------------------------- */
/* This routine fills out the device specific stuff information
 */
static void _te1_set_chip_stuff(STUFF *sptr)
{
	sptr->t1s.mydevice = (DEVICE *) sptr->d->BaseAdr;
	sptr->t1s.mysubc = sptr->subc % MAX_T1_CHANNELS;
	sptr->t1s.myfrmr = (FRAMER *) 
		&sptr->t1s.mydevice->te1[sptr->t1s.mysubc];
	sptr->t1s.myliu = (LIU *) 
		&sptr->t1s.mydevice->bertliu.liu[sptr->t1s.mysubc];
	sptr->t1s.rxslip = 0;
	return;
}

/* This routine performs the utility initialization
 */
static int utility_init(STUFF *sptr, int set_clear)
{
int retval = 1;
FRAMER *f = (FRAMER *)&sptr->t1s.mydevice->te1[0];

	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_UTIL);
		f->gfimr |= (1 << sptr->t1s.mysubc);
		if(!retval)
			return(0);
	}
	else {
		f->gfimr &= ~(1 << sptr->t1s.mysubc);
		retval = hook_isr(sptr, set_clear, INTFLAG_UTIL);
		if(!retval)
			return(0);
	}
			
	return(retval);
}

/*------------------------------------------------------------------------*/
/* The following tables are used to assist signaling
 */

#define FLIP_EM(x) ((((x) & 0x08) >> 3) | (((x) & 0x04) >> 1) | \
	(((x) & 0x02) << 1) | (((x) & 0x01) << 3))

static int flip_table[16] = {
	FLIP_EM(0),
	FLIP_EM(1),
	FLIP_EM(2),
	FLIP_EM(3),
	FLIP_EM(4),
	FLIP_EM(5),
	FLIP_EM(6),
	FLIP_EM(7),
	FLIP_EM(8),
	FLIP_EM(9),
	FLIP_EM(10),
	FLIP_EM(11),
	FLIP_EM(12),
	FLIP_EM(13),
	FLIP_EM(14),
	FLIP_EM(15),
	};

static int rxsf_flip_table[16] = {
	/* ABAB	baBA */
	/* 0000	0000 */		0x0,
	/* 0001 1000 */		0x8,
	/* 0010 0100 */		0x4,
	/* 0011 1100 */		0xc,
	/* 0100 1000 */		0x8,
	/* 0101 0010 */		0x2,
	/* 0110 1100 */		0xc,
	/* 0111 0110 */		0x6,
	/* 1000 0100 */		0x4,
	/* 1001 1100 */		0xc,
	/* 1010 0001 */		0x1,
	/* 1011 1001 */		0x9,
	/* 1100 1100 */		0xc,
	/* 1101 0110 */		0x6,
	/* 1110 1011 */		0xb,
	/* 1111 0011 */		0x3,
	};

static int txsf_flip_table[16] = {
	/* baBA	ABAB */
	/* 0000	0000 */		0x0,
	/* 0001 1010 */		0xa,
	/* 0010 0101 */		0x5,
	/* 0011 1111 */		0xf,
	/* 0100 1000 */		0x8,
	/* 0101 1000 */		0x8,
	/* 0110 1101 */		0xd,
	/* 0111 1101 */		0xd,
	/* 1000 0100 */		0x4,
	/* 1001 1110 */		0xe,
	/* 1010 0100 */		0x4,
	/* 1011 1110 */		0xe,
	/* 1100 1100 */		0xc,
	/* 1101 1100 */		0xc,
	/* 1110 1100 */		0xc,
	/* 1111 1100 */		0xc,
	};


/*------------------------------------------------------------------------*/
/* This will retreive ONE channel of signaling
 * The bits are in dcba 
 */
static int poll_rxbits(STUFF *sptr, int ts, int *rxbits)
{
int retval = 1;
int index = 0;
STANDARD_DECLARE;

	index = (ts > 11) ? (ts - 12) : ts; 

	*rxbits = ((ts > 11) ? (f->rs[index]) : ((f->rs[index] >> 4))) & 0x0f;

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine sets up for signaling 
 */
static int signal_init(STUFF *sptr, int set_clear)
{
int retval = 1;
int i, val;
STANDARD_DECLARE;

	if(set_clear) {
		/* Have the upper levels do the debouncing */
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_SIGNLDEBOUNCE, 1);

		f->tcr1 |= TCR1_TSSE;

		for(i=0;i<MAX_T1_TIMESLOTS;i++) {
			poll_rxbits(sptr, i, &val);
			sptr->ds0[i].rxbits = val;
		}
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as clear channel - no CAS
 */
static int signal_clear(STUFF *sptr, int ts, int set_clear)
{
int retval = 1;
int index,bit;
STANDARD_DECLARE;

	index = ts / 8;
	bit = ts % 8;

	f->ssie[index] &= ~(1 << bit);
	f->tcice[index] &= ~(1 << bit);

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as clear channel - no CAS
 */
static int signal_idle(STUFF *sptr, int ts, int ipat, int set_clear)
{
int retval = 1;
int index, bit;
STANDARD_DECLARE;

	if(ipat != (-1)) {
		f->tidr[ts] = ipat;
	}

	index = ts / 8;
	bit = ts % 8;

	if(set_clear) {
		f->tcice[index] |= (1 << bit);
	} else {
		f->tcice[index] &= ~(1 << bit);
	}

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine set a voice channel as needing CAS processing
 */
static int signal_rbcas(STUFF *sptr, int ts, int set_clear)
{
int retval = 1;
int index,bit;
STANDARD_DECLARE;

	index = ts / 8;
	bit = ts % 8;

	f->ssie[index] |= (1 << bit);
	f->tcice[index] &= ~(1 << bit);

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This routine sets the transmit signaling bits for a CAS channel
 */
static int signal_txbits(STUFF *sptr, int ts, unsigned char dcba)
{
int retval = 1;
int bits, index;
STANDARD_DECLARE;

	index = (ts > 11) ? (ts - 12) : ts; 
	bits = f->ts[index];

	if(sptr->cfg.framing == FF_ESF)
		dcba = flip_table[dcba & 0x0f];
	else
		dcba = txsf_flip_table[dcba & 0x0f];

	bits &= (ts > 11) ? 0xf0 : 0x0f;

	bits |= (ts > 11) ? dcba : (dcba << 4);

	f->ts[index] = bits;

	return(retval);

}

/*------------------------------------------------------------------------*/
/* This will poll alarms on the appriate line
 */
static int poll_alarm(STUFF *sptr, int alarm_type, int *valueptr)
{
int retval = 1;
int value = 0;
STANDARD_DECLARE;

	switch(alarm_type) {
	case TMS_RXLOS:

		value = (f->rrts1 & RRTS1_RLOS) ? 1 : 0;
		break;

	case TMS_RXOOF:

		value = (f->rrts1 & RRTS1_RLOF) ? 1 : 0;
		break;

	case TMS_RXAIS:

		value = (f->rrts1 & RRTS1_RAIS) ? 1 : 0;
		break;

	case TMS_RXRAI:

		value = (f->rrts1 & RRTS1_RRAI) ? 1 : 0;
		break;

	default:
		retval = 0;
		break;
	}

	*valueptr = value;
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine will retreive the loop information
 */
static int poll_loop(STUFF *sptr, int loop_type, int *set_clear)
{
int retval = 1;
int value = 0;
LIU *l = sptr->t1s.myliu;
STANDARD_DECLARE;

	switch(loop_type) {
	case LOOP_REMSWITCH:
		/* Remote is when the the receive signal is looped back
		 * at the analog layer to the remote device.
		 * This is also known as line loopback
		 */

		value = (f->rcr3 & RCR3_RLB) ? 1 : 0;

		break;

	case LOOP_LOCSWITCH:
		/* Local Loopback is when the voice channels are looped
		 * before the framer section.
		 */

		value = (l->lmcr & LMCR_LLB) ? 1 : 0;

		break;

	case LOOP_PAYSWITCH:
		/* Payload Loopback is when the framer receives the signal
		 * then, after the framer, the voice channels are re-sent out
		 * and the overhead is re-generated.
		 */

		value = (f->rcr3 & RCR3_PLB) ? 1 : 0;

		break;

	case LOOP_FRMSWITCH:	
		/* Framer Loopback is when the signal is set out of
		 * the framer and looped back to the framer but
		 * before it hits the LIU.  You will send and AIS while
		 * this happens.  The AIS is done by the chip and not the
		 * Upper layers.
		 */

		value = (f->rcr3 & RCR3_FLB) ? 1 : 0;

		break;

	case LOOP_REMRXUPCODE:

		value = (f->rrts3 & RRTS3_LUP) ? 1 : 0;
		break;

	case LOOP_REMRXDNCODE:

		value = (f->rrts3 & RRTS3_LDN) ? 1 : 0;
		break;

	default:
		retval = 0;
		break;
	}

	*set_clear = value;
	return(retval);
}

/*------------------------------------------------------------------------*/
/* This will retreive ALL 24 channels of signaling
 * It will fill the array of all_rxbits with all signaling channels
 * all_rxbits must be 24 ints long.
 */
static int poll_allrxbits(STUFF *sptr, int *all_rxbits)
{
int retval = 1;
int i, val, bits;
int *p;
STANDARD_DECLARE;

	p = all_rxbits;

	for(i = 0; i < MAX_T1_TIMESLOTS/2; i++) {
		val = f->rs[i];

		bits = sptr->ds0[i + 12].rxbits = val & 0x0f;

		if(sptr->cfg.framing == FF_ESF)
			bits = flip_table[bits & 0x0f];
		else
			bits = rxsf_flip_table[bits & 0x0f];

		*(p+i+12) = bits;

		bits = sptr->ds0[i].rxbits = (val >> 4) & 0xff;

		if(sptr->cfg.framing == FF_ESF)
			bits = flip_table[bits & 0x0f];
		else
			bits = rxsf_flip_table[bits & 0x0f];

		*(p+i) = bits;

	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine will retreive the performance data
 */
static int get_pm(STUFF *sptr, T1_PERFINFO *pm)
{
int retval = 1;
STANDARD_DECLARE;

	pm->CVcount = ((f->lcvcr1 << 8) | f->lcvcr2);

	if(sptr->cfg.framing == FF_ESF) {
		pm->CRCcount = ((f->pcvcr1 << 8) | f->pcvcr2);
		pm->FEcount = ((f->foscr1 << 8) | f->foscr2);
	}
	else {
		pm->CRCcount = 0;
		pm->FEcount = ((f->pcvcr1 << 8) | f->pcvcr2);
	}

	pm->EBcount = 0;
	pm->SLevent = sptr->t1s.rxslip;
	sptr->t1s.rxslip = 0;

	pm->SEFEevent = (f->rls2 & RLS2_SEFE) ? 1 : 0;
	f->rls2 |= RLS2_SEFE;

	return(retval);


}

/*------------------------------------------------------------------------*/
/* This will transmit on the appriate line 
 */
static int transmit_alarm(STUFF *sptr, int alarm_type, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;

	switch(alarm_type) {
	case TMS_TXRAI:
		if(set_clear) {
			f->tcr1 |= TCR1_TRAI;
		}
		else {
			f->tcr1 &= ~TCR1_TRAI;
		}
		break;
	case TMS_TXAIS:
		if(set_clear) {
			f->tcr1 |= TCR1_TAIS;
		}
		else {
			f->tcr1 &= ~TCR1_TAIS;
		}
		break;

	default:
		retval = 0;
		break;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable t1231 processing on the line 
 */
static int t1231_init(STUFF *sptr, int set_clear)
{
int retval = 1;

	/* Nothing to do here */

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable one sec processing on the approriate line
 */
static int onesec_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;


	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_ONESEC);
		if(!retval)
			return(0);
		f->rim4 |= RIM4_TIMER;
	}
	else {
		f->rim4 &= ~RIM4_TIMER;
		retval = hook_isr(sptr, set_clear, INTFLAG_ONESEC);
		if(!retval)
			return(0);
	}
	return(retval);
}

#if (MAX_GLOBAL_TICKERS	> 0)

/* -------------------------------------------------------------------------- */
/* The device has global tickers associated with it so,
 * the following routine is called by the global file for processing of
 * the global tickers
 */
static void T1GlobalTickerHandler(GDEVICE *d, int ticker)
{
int i;
STUFF *sptr;

	switch(ticker) {
	case GLOBAL_ALARM_TICKER:
		for(i = 0; i < MAX_T1_CHANNELS; i++) {

			if (!((sptr = &T1stuff[d->id][i])->init))
				continue;	
		}
		break;

	default:
		break;
	}
}


/*------------------------------------------------------------------------*/
/* This routine is called by global when the first time a ticker is
 * registered.  This routine makes the call back to the approprate TMS
 * to get the ticker hooked up.
 */
static void T1RegTicker(STUFF *sptr, void (*global_handler)(),
	int on_off, int ticker, int ms, void *handle)
{

	if(on_off)
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_HOOKOSTIC, ms, handle,
			global_handler, sptr->d, ticker);
		
	else
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_UNHOOKOSTIC, 
			*((unsigned long *)(handle)));
}

/*------------------------------------------------------------------------*/
/* This routine start a global ticker going
 */
static void start_global_ticker(STUFF *sptr, int ticker, int onoff)
{

	NCID_GHOOKTICKER(sptr->d, DRVR_IDX_T1, ticker, onoff,
		T1RegTicker, sptr, T1GlobalTickerHandler);
}

#endif	/* MAX_GLOBAL_TICKERS */

/*------------------------------------------------------------------------*/
/* This routine enables alarms 
 * Alarms:
 *         LOS OOF AIS RAI
 */
static int alarm_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;

	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_ALM);
		if(!retval)
			return(0);

		f->rim1 |= RIM1_RRAIC | RIM1_RAISC | RIM1_RLOSC |
			RIM1_RLOFC | RIM1_RRAID | RIM1_RAISD |
			RIM1_RLOSD | RIM1_RLOFD;
	}
	else {
		f->rim1 &= ~(RIM1_RRAIC | RIM1_RAISC | RIM1_RLOSC |
			RIM1_RLOFC | RIM1_RRAID | RIM1_RAISD |
			RIM1_RLOSD | RIM1_RLOFD);

		retval = hook_isr(sptr, set_clear, INTFLAG_ALM);
		if(!retval)
			return(0);
	}
			
	return(retval);
}


/*------------------------------------------------------------------------*/
/* Helper routine used to enable/disable transmission of loop codes
 */

static void txloop(STUFF *sptr, int enabled, int code, int bitcount)
{
int tc;
STANDARD_DECLARE;

	if (!enabled) {
		f->tcr3 &= ~TCR3_TLOOP;
		return;
	}

	code = (code << (8 - bitcount)) & 0xff;

	switch (bitcount) {
	   case 1: 
		code |= (code >> 1);
	   case 2: /* fall through intended */
		code |= (code >> 2);
	   case 4:	/* fall through intended */
		code |= (code >> 4);
	   case 8:
		tc = 3;
		break;
	   case 3: 
		code |= (code >> 3);
	   case 6:	/* fall through intended */
		tc = 1;
		break;
	   case 7:
		tc = 2;
		break;
	   case 5: 	/* fall through intended */
	   default:
		tc = 0;
		break;
	}

	f->tcd1 = code;
	f->tcd2 = code;

	f->tcr4 = (UC)((f->tcr4 & (~TCR4_TC(3))) | TCR4_TC(tc));

	f->tcr3 |= TCR3_TLOOP;
}

/*------------------------------------------------------------------------*/
/* This routine sets up loopbacks of the indicated type
 */

static int do_loopback(STUFF *sptr, int looptype, int set_clear, 
	int bits, int code)
{
int retval = 1;
LIU *l = sptr->t1s.myliu;
STANDARD_DECLARE;

	switch(looptype) {
	case LOOP_REMSWITCH:
		/* Remote is when the the receive signal is looped back
		 * at the analog layer to the remote device.
		 * This is also known as line loopback
		 */

		if(set_clear) {
			f->rcr3 |= RCR3_RLB;
		}
		else {
			f->rcr3 &= ~RCR3_RLB;
		}

		break;

	case LOOP_LOCSWITCH:
		/* Local Loopback is when the voice channels are looped
		 * before the framer section.
		 */
		if(set_clear) {
			l->lmcr |= LMCR_LLB;
		}
		else {
			l->lmcr &= ~LMCR_LLB;
		}

		break;

	case LOOP_PAYSWITCH:
		/* Payload Loopback is when the framer receives the signal
		 * then, after the framer, the voice channels are re-sent out
		 * and the overhead is re-generated.
		 */
		if(set_clear) {
			f->rcr3 |= RCR3_PLB;
		}
		else {
			f->rcr3 &= ~RCR3_PLB;
		}
		break;

	case LOOP_FRMSWITCH:	
		/* Framer Loopback is when the signal is set out of
		 * the framer and looped back to the framer but
		 * before it hits the LIU.  You will send and AIS while
		 * this happens.  The AIS is done by the chip and not the
		 * Upper layers.
		 */


		if(set_clear) {
			f->rcr3 |= RCR3_FLB;
		}
		else {
			f->rcr3 &= ~RCR3_FLB;
		}
		break;

	case LOOP_REMTXUPCODE:
	case LOOP_REMTXDNCODE:
		txloop(sptr, set_clear, code, bits);
		break;

	case LOOP_REMRXUPCODE:
	case LOOP_REMRXDNCODE:
		code = (code << (8 - bits)) & 0xff;
		switch(bits) {
		   case 1:
			code = code | (code >> 1);
		   case 2:	/* FALL THROUGH INTENDED */
			code = code | (code >> 2);
		   case 4:	/* FALL THROUGH INTENDED */
			code = code | (code >> 4);
		   case 8:	/* FALL THROUGH INTENDED */
			bits = 8;
			break;
		   case 5:
		   case 7:
			break;
		   case 3:
			code = code | (code >> 3);
		   case 6:	/* FALL THROUGH INTENDED */
			bits = 6;
			break;
		}
		if(looptype == LOOP_REMRXUPCODE) {
			f->rupcd2 = f->rupcd1 = code;
	
			f->ribcc_rcr2 = (UC)((f->ribcc_rcr2 & 
				(~(RIBCC_RUP(7)))) | RIBCC_RUP(bits - 1));
		}
		else {

			f->rdncd2 = f->rdncd1 = code;

			f->ribcc_rcr2 = (UC)((f->ribcc_rcr2 & 
				(~(RIBCC_RDN(7)))) | RIBCC_RDN(bits - 1));
		}

		break;

	default:
		retval = 0;
		break;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine turns on and off loop detection 
 */
static int loop_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;


	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_LOOP);
		if(!retval)
			return(0);
		f->rim3 |= RIM3_LDND | RIM3_LUPD | RIM3_LDNC | RIM3_LUPC;
	}
	else {
		retval = hook_isr(sptr, set_clear, INTFLAG_LOOP);
		if(!retval)
			return(0);
		f->rim3 &= ~(RIM3_LDND | RIM3_LUPD | RIM3_LDNC | RIM3_LUPC);
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* This routine sets up the clock used for the transmitter 
 */
static int set_txclock(STUFF *sptr, CLOCK_TYPE txclk)
{
int retval = 1;
STANDARD_DECLARE;

	switch(txclk) {
	case SYSCLOCK:	/* clock from central timing source */
		/* The TCLK pin */
		f->tcr3 = (f->tcr3 & (~TCR3_TCSS(3))) | TCR3_TCSS(0);
		break;

	case RXCLOCK:	/* loop timing selection */
		f->tcr3 = (f->tcr3 & (~TCR3_TCSS(3))) | TCR3_TCSS(3);
		break;

	case LOCALOSC:	/* clock from a local oscillator */
	case OTHERCLK:	/* some other clock selection */
	default:
		retval = 0;
		break;
	}
	
	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable T1.403 processing
 */
static int t1403_init(STUFF *sptr, int set_clear)
{
int retval = 1 , i;
STANDARD_DECLARE;

	if(set_clear) {
		retval = hook_isr(sptr, set_clear, INTFLAG_T1403);
		if(!retval)
			return(0);
		

		/* Reset the HDLC channels */
		f->rhc |= RHC_RHR;
		f->rhc &= ~RHC_RHR;
		f->thc1 |= THC1_THR;
		f->thc1 &= ~THC1_THR;

		f->rbocc |= RBOCC_RBR;
		/* wait for the reset bit to clear */
		for(i = 0; i < MAX_RESET_WAIT; i++)
			if(!(f->rbocc & RBOCC_RBR)) break;

		if( i >= MAX_RESET_WAIT) {
			/* Reset should have finished by now, 
			 * this is an error */
			(*sptr->cback)(sptr->lnPtr, TE1DCLBK_FATALERROR, 0);
		}
		
		f->rbocc = RBOCC_RBD(1) | RBOCC_RBF(1);
		f->txpc = TXPC_THPAMS | TXPC_THPAEN;
		f->rhfc = RHFC_RFHWM(HIGH_WATER_MARK);
		f->thfc = THFC_TFLWM(LOW_WATER_MARK);
		f->rim5 |= RIM5_RPS;	
		f->thc1 = THC1_THMS;
		f->thc2 |= THC2_THCEN;

	}
	else {

		f->rhc |= RHC_RHR;
		f->thc1 |= THC1_THR;
		f->thc2 &= ~THC2_THCEN;

		retval = hook_isr(sptr, set_clear, INTFLAG_T1403);
		if(!retval)
			return(0);
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* processes sending hdlc packets.
 * The packet has already been copied into the sptr area 
 * The flag = 1 means this is the start of the send
 * if flag = 0, means that this is a subsequent part of the packet
 * if flag = 2, means turn off the HDLC packet facility
 */
static int send_fdl_packet(STUFF *sptr, int flag)
{
int retval = 1;
STANDARD_DECLARE;

	switch(flag) {
	case 0:
		break;
	case 1:
		sptr->t1s.sendingBOC = 0;

		f->thc2 &= ~(THC2_SBOC);
		f->tcr2 |= TCR2_TFDLS;
		sptr->t1s.txremaining = sptr->pkttx.byte_count;
		sptr->t1s.sendingPKT = 1;
		hdlc_xmit(sptr);

		break;
	case 2:
		f->tim2 = 0;
		f->rim7 &= ~(RIM7_BC | RIM7_BD);
		f->thc2 &= ~(THC2_SBOC);
		f->rhc = 0;

		f->thc1 = 0;
		f->tcr2 &= ~TCR2_TFDLS;
		f->rhc = RHC_RHR | RHC_RHMS;

		f->thc1 = THC1_THR | THC1_THMS;
		break;
	default:
		retval = 0;
		break;
	}
	return(retval);
}

/*------------------------------------------------------------------------*/
/* processes sending BOC codes
 * the boc code and the action has already been coped into the
 * sptr area
 */
static int send_fdl_boc(STUFF *sptr)
{
int retval = 1;
STANDARD_DECLARE;

	if(sptr->boctx.active) {
		if(sptr->t1s.sendingPKT) {
		   unsigned long imask;

			imask = INTS_OFF();

			f->tim2 &= 
			  ~(TIM2_TLWMS | TIM2_TNFS | TIM2_TMEND);

			f->thc1 = THC1_THR | THC1_THMS;
			f->thc1 = THC1_THMS;

			INTS_ON(imask);
		}
			
		sptr->t1s.sendingPKT = 0;
		sptr->t1s.txremaining = 0;
		sptr->t1s.sendingBOC = 1;

		/* send the BOC
		 */

		f->tcr2 &= ~TCR2_TFDLS;
		f->tboc = (sptr->boctx.boc >> 1);
		f->thc2 |= THC2_SBOC;

	}
	else {
		sptr->t1s.sendingBOC = 0;
		f->thc2 &= ~(THC2_SBOC);
	}
	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable TR-54016 processing
 */
static int tr54016_init(STUFF *sptr, int set_clear)
{
int retval = 1;

	/* Nothing to do on DS26528 */

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable TR-00008 processing
 * This prepares the device to start tx and receive of TR-0008 items.
 */
static int tr008_init(STUFF *sptr, int set_clear)
{
int retval = 1;

	if(set_clear) {
		/* !!! Enable TR-00008 processing here !!! */
	}
	else {
		/* !!! Diable TR-00008 processing here !!! */
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable EXZ processing
 * This enables counting BPVs and EXZ at the same time
 * When disabled, EXZ are not counted.  Used for the BPV alarm
 * during TR-00008 applications, and enabled other times.
 */
static int exz_init(STUFF *sptr, int set_clear)
{
int retval = 1;
STANDARD_DECLARE;


	if(set_clear) {
		f->ercnt |= ERCNT_LCVCRF;
	}
	else {
		f->ercnt &= ~(ERCNT_LCVCRF);
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Enable the CMAS bit processing for TR-00008 applications
 * CMAS_C = 0x01 =concentration field
 * CMAS_M = 0x02 =Maintenance field
 * CMAS_A = 0x04 =Alarm field
 * CMAS_S = 0x08 =Protection Switching field
 */
static int cmas_init(STUFF *sptr, int cmas)
{
int retval = 1;

	if(cmas & 0x01) {
		/* !!! Enable the concentration bits here !!! */
	}
	else {
		/* !!! Disable the concentration bits here !!! */
	}

	if(cmas & 0x02) {
		/* !!! Enable the maintenance bits here !!! */
	}
	else {
		/* !!! Disable the maintenance bits here !!! */
	}

	if(cmas & 0x04) {
		/* !!! Enable the Alarm bits here !!! */
	}
	else {
		/* !!! Disable the Alarm bits here !!! */
	}

	if(cmas & 0x08) {
		/* !!! Enable the Protection Switching bits here !!! */
	}
	else {
		/* !!! Disable the Protection Switching bits here !!! */
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* 
 * This routine sets up the configration of the channel
 * sptr is the pointer to the stuff structure
 */
static int _t1Config(STUFF *sptr)
{
int retval = 1;
unsigned char value2 = 0;
LIU *l = sptr->t1s.myliu;
STANDARD_DECLARE;


	switch(sptr->cfg.framing) {
	case FF_D4:
		f->tcr3 |= TCR3_TFM;
		f->rcr1 |= RCR1_RFM;
		f->tcr2 &= ~TCR2_TFDLS;
		f->tfdl = 0x1c;
		break;

	case FF_ESF:
		f->tcr3 &= ~TCR3_TFM;
		f->rcr1 &= ~RCR1_RFM;
		f->tcr2 |= TCR2_TFDLS;
		f->tfdl = 0x7e;
		break;

	case FF_SLC96:
		f->tcr2 |= TCR2_TSLC96;
		break;

	default:
		return(0);
	}

	switch(sptr->cfg.coding) {
	case LC_B8ZS:
		f->tcr1 |= TCR1_TB8ZS;
		f->rcr1 |= RCR1_RB8ZS;
		f->tcr2 &= ~TCR2_PDE;
		break;

	case LC_AMI:
		f->tcr1 &= ~TCR1_TB8ZS;
		f->rcr1 &= ~RCR1_RB8ZS;
		f->tcr2 |= TCR2_PDE;
		break;

	default:
		return(0);
	}

	/* Handle Line Build Out */


	if(sptr->cfg.lbo == -1) {	/* 0 db LH */
		value2 = 0;
	}
	else if(sptr->cfg.lbo == -2) {	/* -7.5 db LH */
		value2 = 5;
	}
	else if(sptr->cfg.lbo == -3) {	/* -15 db LH */
		value2 = 6;
	}
	else if(sptr->cfg.lbo == -4) {	/* -22.5 db LH */
		value2 = 7;
	}
	else if(sptr->cfg.lbo <= 133) {
		value2 = 0;
	}
	else if(sptr->cfg.lbo <= 266) {
		value2 = 1;
	}
	else if(sptr->cfg.lbo <= 399) {
		value2 = 2;
	}
	else if(sptr->cfg.lbo <= 533) {
		value2 = 3;
	}
	else if(sptr->cfg.lbo <= 655) {
		value2 = 4;
	}

	l->ltitsr = (l->ltitsr & 0xf1) | value2;

	if(sptr->cfg.lbo < 0) {
		/* long haul */
		l->lrismr = (l->lrismr & 0x03) | LRISMR_RSMS(1);
	} else {
		/* short haul */
		l->lrismr = (l->lrismr & 0x03) | LRISMR_RSMS(3);
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* 
 * This routine resets the T1 portion of the chip.
 * sptr is the pointer to the stuff structure
 */
static int _te1Reset(STUFF *sptr)
{
int retval = 1;
LIU *l = sptr->t1s.myliu;
STANDARD_DECLARE;

	if(!sptr->greset){
		NCID_GRESET(sptr->d);
		sptr->greset = 1;
	}

	utility_init(sptr, (sptr->utility_init = 0));
	alarm_init(sptr, (sptr->alarm_init = 0));
	onesec_init(sptr, (sptr->onesec_init = 0));
	t1403_init(sptr, (sptr->t1403_init = 0));
	loop_init(sptr, (sptr->loop_init = 0));

	/* Soft Reset into T1 mode */

	f->tmmr = TMMR_SFTRST;
	f->tmmr = 0;
	
	f->rmmr = RMMR_SFTRST;
	f->rmmr = 0;

	f->tmmr = TMMR_FRM_EN;
	f->rmmr = RMMR_FRM_EN;
	
	f->rcr1 = RCR1_SYNCC |
		((RESYNC_CRITERIA | RX_UNIPOLAR) ? RCR1_ARC : 0) |
		((SYNC_TIME) ? RCR1_SYNCT : 0);

	f->rcr3 = (RX_UNIPOLAR) ? RCR3_IDF : 0;

	f->riocr = ((RSYNC_MODE1) ? RIOCR_RSMS1 : 0) |
		((RSYNC_MODE2_T1) ? RIOCR_RSMS2 : 0) |
		((RSYNC_IO_SELECT) ? RIOCR_RSIO : 0) |
		((RSYNC_SKIP) ? RIOCR_RSMS : 0) |
		((RSYNC_CLK_MODE_T1) ? RIOCR_RSCLKM : 0) |
		((H100_MODE) ? RIOCR_H100EN : 0) |
		((RSYNC_INV) ? RIOCR_RSYNCINV : 0) |
		((RCLK_INV) ? RIOCR_RCLKINV : 0);

	f->rescr = ((RX_ELASTIC_STORE_EN) ? RESCR_RESE : 0) |
		((RES_MIN_DELAY) ? RESCR_RESMDM : 0) |
		((RXS_ZONE_SEL) ? RESCR_RSZS : 0) |
		((GAP_CLOCK_EN) ? RESCR_RGCLKEN : 0) |
		((RX_CHAN_FORMAT) ? RESCR_RDATFMT : 0);

	f->ercnt =
		((MOS_COUNT_SELECT) ? ERCNT_MOSCRF : 0) |
		((PCVCR_SELECT) ? ERCNT_FSBE : 0) |
		((ERROR_ACU_MODE) ? ERCNT_EAMS : 0) |
		((UPDATE_SELECT_T1) ? ERCNT_ECUS : 0) |
		((MANUAL_UPDATE_SEL) ? ERCNT_MCUS : 0) |
		((ONE_SECOND_SEL) ? ERCNT_1SECS : 0);

	f->riboc = (RIBOC_DA(DEVICE_ASSIGNMENT)) |
		((IBO_EN) ? RIBOC_IBOEN : 0) |
		((IBO_SEL) ? RIBOC_IBOSEL : 0) |
		(RIBOC_IBS(IBO_SIZE));
	
	f->tiboc = (TIBOC_DA(DEVICE_ASSIGNMENT)) |
		((IBO_EN) ? TIBOC_IBOEN : 0) |
		((IBO_SEL) ? TIBOC_IBOSEL : 0) |
		(TIBOC_IBS(IBO_SIZE));
	
	f->tcr3 = (TX_UNIPOLAR) ? TCR3_ODF : 0;

	f->tiocr = ((TSYNC_MODE) ? TIOCR_TSM : 0) |
		((TSYNC_DOUBLE_WIDE) ? TIOCR_TSDW : 0) |
		((TSYNC_IO) ? TIOCR_TSIO : 0) |
		((TSSYNC_MODE) ? TIOCR_TSSM : 0) |
		((TSYSCLK_MODE_T1) ? TIOCR_TSCLKM : 0) |
		((TSYNCIO_INV) ? TIOCR_TSSYNCINV : 0) |
		((TSYNC_INV) ? TIOCR_TSYNCINV : 0) |
		((TCLK_INV) ? TIOCR_TCLKINV : 0);

	f->tescr = ((TX_ELASTIC_STORE_EN) ? TESCR_TESE : 0) |
		((TES_MIN_DELAY) ? TESCR_TESMDM : 0) |
		((TXS_ZONE_SEL) ? TESCR_TSZS : 0) |
		((TDATA_EN) ? TESCR_TDATAEN : 0) |
		((TGAP_CLOCK_EN) ? TESCR_TGCLKEN : 0) |
		((TX_CHAN_FORMAT) ? TESCR_TDATFMT : 0);


	l->ltrcr = ((JAT_DEPTH) ? LTRCR_JADS : 0) |
		(LTRCR_JAPS(JAT_POSITION)) |
		LTRCR_T1J1E1 |
		((LOS_CRITERIA) ? LTRCR_LSC : 0);

	l->ltitsr = ((TXTERM_OFF) ? LTITSR_TIMPTOFF : 0) |
			(LTITSR_TIMPL(1));
	
	l->lrismr = ((RXTERM_OFF) ? LRISMR_RIMPOFF : 0) |
			(LRISMR_RSMS(1));


	retval = _t1Config(sptr);
	if(!retval)
		return(0);

	retval = transmit_alarm(sptr, TMS_TXAIS, 1);

	/* after the configuration turn on the framer */
	f->tmmr |= TMMR_INIT_DONE;
	f->rmmr |= RMMR_INIT_DONE;

	/* Turn on LIU output */
	l->lmcr = LMCR_TE;

	return(retval);
}


/*------------------------------------------------------------------------*/
/* 
 * This routine UN resets the T1 portion of the chip.
 * sptr is the pointer to the stuff structure
 */
static int _te1UNReset(STUFF *sptr)
{
int retval = 1;

	/* disable all interrupts
	 */

	utility_init(sptr, (sptr->utility_init = 0));
	alarm_init(sptr, (sptr->alarm_init = 0));
	onesec_init(sptr, (sptr->onesec_init = 0));
	t1403_init(sptr, (sptr->t1403_init = 0));
	loop_init(sptr, (sptr->loop_init = 0));

	retval = _te1Reset(sptr);
	if(!retval)
		return(0);

	retval = transmit_alarm(sptr, TMS_TXAIS, 1);

	if(sptr->greset) {
		NCID_GUNRESET(sptr->d);
		sptr->greset = 0;
	}

	return(retval);
}

/* -------------------------------------------------------------------------- */
/* Set the USR_SIDE vs NET_SIDE indication.
 */
 
static int set_side(STUFF *sptr, int side)
{
int retval = 1;

	/* Nothing to do */

	return(retval);
}

/* -------------------------------------------------------------------------- */
/* Process any polling operations here 
 */
static int process_OSTick(STUFF *sptr, int ticker)
{
int retval = 1;

	return(retval);
}

/*------------------------------------------------------------------------*/
static void process_SR7_interrupt(STUFF *sptr)
{
volatile unsigned char status,data;
STANDARD_DECLARE;

	status = f->rls7; 

	if(status & RLS7_BD) {
		data = (f->rboc & 0x3f) << 1;
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_T1403RXBOC, data);
	}
	if(status & RLS7_BC) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_T1403RXBOC, 0xff);

	}

	f->rls7 = status;
}

/*------------------------------------------------------------------------*/
static void process_SR1_interrupt(STUFF *sptr)
{
volatile unsigned char status, statusl;
STANDARD_DECLARE;

	status = f->rrts1;
	statusl = f->rls1;

	if(statusl & (RLS1_RRAIC | RLS1_RRAID)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXRAI, 
			       (status & RRTS1_RRAI) ? 1 : 0);
	}
	
	if(statusl & (RLS1_RAISC | RLS1_RAISD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXAIS, 
			       (status & RRTS1_RAIS) ? 1 : 0);
	}

	if(statusl & (RLS1_RLOSC | RLS1_RLOSD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXLOS, 
			       (status & RRTS1_RLOS) ? 1 : 0);
	}

	if(statusl & (RLS1_RLOFC | RLS1_RLOFD)) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ALARMRX, TMS_RXOOF, 
			       (status & RRTS1_RLOF) ? 1 : 0);
	}

	f->rls1 = statusl;


}

/*------------------------------------------------------------------------*/
static void process_SR4_interrupt(STUFF *sptr)
{
volatile unsigned char statusl;
STANDARD_DECLARE;

	statusl = f->rls4;
	
	if(statusl & RLS4_TIMER) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_ONESEC);
	}

	if(statusl & RLS4_RSLIP) {
		sptr->t1s.rxslip = 1;
	}

	f->rls4 = statusl;
}

/*------------------------------------------------------------------------*/
static void process_SR3_interrupt(STUFF *sptr)
{
volatile unsigned char status, statusl;
STANDARD_DECLARE;

	status = f->rrts3;
	statusl = f->rls3;


	if(statusl & (RLS3_LUPD | RLS3_LUPC)) {
		(*sptr->cback)(sptr->lnPtr, 
			TE1DCLBK_LOOP, 
			LOOP_REMRXUPCODE, (status & RRTS3_LUP) ? 1 : 0,
			sptr->rxlup.code, sptr->rxlup.bits);
	} 

	if(statusl & (RLS3_LDND | RLS3_LDNC)) {
		(*sptr->cback)(sptr->lnPtr, 
			TE1DCLBK_LOOP, 
			LOOP_REMRXDNCODE, (status & RRTS3_LDN) ? 1 : 0,
			sptr->rxldn.code, sptr->rxldn.bits);
	} 
	
	f->rls3 = statusl;
}



/*------------------------------------------------------------------------*/
/* processes transmission of HDLC packets
 * h = the hdlc channel number
 */
static void hdlc_xmit(STUFF *sptr)
{
int count, txroom;
STANDARD_DECLARE;

	if(sptr->t1s.txremaining == 0) {
		f->tim2 &= ~(TIM2_TMEND | TIM2_TLWMS | TIM2_TNFS);
		return;
	}

	txroom = f->tfba & 0x7f;
	if(txroom == 0) 
		txroom = (FIFOSIZE-1);
	if(txroom >=128) 
		txroom = (FIFOSIZE-1);

	/* never fill the fifo up since we can not tell no room from all room */

	count = (sptr->t1s.txremaining > txroom) ? 
		txroom : sptr->t1s.txremaining;

	f->thc1 &= ~(THC1_TEOM);

	while(count--) {
		if (sptr->t1s.txremaining == 1)
			f->thc1 |= THC1_TEOM;

		f->thf = sptr->pkttx.pkt[sptr->pkttx.byte_count -
						   sptr->t1s.txremaining--];
	}

	if(sptr->t1s.txremaining == 0) {
		f->tim2 = (UC)((f->tim2 & ~(TIM2_TLWMS))
		 | TIM2_TMEND);
	}
	else {
		f->tim2 = (UC)((f->tim2 & ~(TIM2_TMEND))
		 | (TIM2_TLWMS));
	}
}

/*------------------------------------------------------------------------*/
/* Handles the HDLC interrupts
 */
static void process_RXHDLC_interrupt(STUFF *sptr)
{
volatile unsigned char status;
volatile unsigned char regval;
int i,cnt;
int packetstart = 0;
unsigned char data;
STANDARD_DECLARE;


	status = f->rls5;
	f->rls5 = status;

	status &= f->rim5;

	if (!status) 
		return;

	/* Check for Packet received
	 */

	if (status & RLS5_RPS) {
		/* packet has started */
		sptr->pktrx.byte_count = 0;
		packetstart = 1;
		f->rim5 = (UC)((f->rim5 & ~(RIM5_RPS)) |
			(RIM5_RPE | RIM5_RHWMS));
	}

	if((status & (RLS5_RPE | RLS5_RHWMS)) || packetstart) {

	   cnt = f->rhpba;
	   if(cnt & RHPBA_MS) {
		if((cnt & 0x7f) == 0) {
			cnt = 128;
		}
		else {
			cnt &= 0x7f;
		}
	   }

	   if ((sptr->pktrx.byte_count + cnt) >= MAX_PACKET) {
		for(i = 0;i < cnt; i++) {
		   	data = f->rhf;
		}
		sptr->pktrx.byte_count = 0;
		return;
	   }
	
	   /* have good data in the fifo */

	   for(i = 0;i < cnt; i++) { 

	   	sptr->pktrx.pkt[sptr->pktrx.byte_count++] = f->rhf;

	   	regval = f->rrts5;

	   	switch(regval & RRTS5_PS(0xff)) {
		   case 2:	/* values 2 through 5 represent errors */
		   case 3:	/* if any of the occur, drop the packet */
		   case 4:
		   case 5:
		   	/* packet being received is no good */
			/* flush it */

		   	for(i = 0; i < cnt; i++) {
		   		data = f->rhf;
			}
			sptr->pktrx.byte_count = 0;
			return;	
		   default:
			break;
	   	}

	   }
		
	   if(status & RLS5_RPE) {
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_T1403RXPKT,
				sptr->pktrx.byte_count, 
				sptr->pktrx.pkt);
	   	sptr->pktrx.byte_count = 0;
	   	f->rim5 = (UC)((f->rim5 & 
	   		~(RIM5_RPE | RIM5_RHWMS)) | (RIM5_RPS));
	   } 
	}
}
/*------------------------------------------------------------------------*/
/* Handles the HDLC interrupts
 */
static void process_TXHDLC_interrupt(STUFF *sptr)
{
volatile unsigned char status;
STANDARD_DECLARE;


	status = f->tls2;
	f->tls2 = status;

	if (sptr->t1s.sendingBOC || !sptr->t1s.sendingPKT) {
		return;
	}

	if (status & TLS2_TMEND) {
		sptr->t1s.sendingPKT = 0;
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_T1403TXPKTCOMPLETE);
		hdlc_xmit(sptr);
		return;
	}

	if (status & (TLS2_TLWMS)) {

		if (status & TLS2_TUDR) {
			/* Setup for a retry
			 */
			/* clear out tx */
			f->thc1 = THC1_THR | THC1_THMS;
			f->thc1 = THC1_THMS;
			sptr->t1s.txremaining = sptr->pkttx.byte_count;
		}
		hdlc_xmit(sptr);
	}
}


/* -------------------------------------------------------------------------- */
/* This is where the real T1 channel processing for an interrupt is done
 */
static int process_t1(STUFF *sptr)
{
int check_again = 0;
unsigned int riir, tiir;
STANDARD_DECLARE;

	riir = (f->riir & 0x5D);
	tiir = (f->tiir & 0x02);

	if(riir & 0x01) {
		process_SR1_interrupt(sptr);
		check_again = 1;
	}
	if(riir & 0x04) {
		process_SR3_interrupt(sptr);
		check_again = 1;
	}
	if(riir & 0x08) {
		process_SR4_interrupt(sptr);
		check_again = 1;
	}
	if(riir & 0x10) {
		process_RXHDLC_interrupt(sptr);
		check_again = 1;
	}
	if(riir & 0x40) {
		process_SR7_interrupt(sptr);
		check_again = 1;
	}
	if(tiir & 0x02) {
		process_TXHDLC_interrupt(sptr);
		check_again = 1;
	}

	return(check_again);
}

/* -------------------------------------------------------------------------- */
/* This is the interrupt service routine for the T1 functions 
 * it gets called by the global routine when the T1 interrupt is hooked up 
 * d points to the global device record.
 */
static int T1_handler(GDEVICE *d)
{
int check_again = 0;
int i;
int id;
STUFF *sptr;
unsigned char fisr, fmask;

	id = d->id;
	fisr = ((DEVICE *) d->BaseAdr)->te1[0].gfisr;
	fmask = ((DEVICE *) d->BaseAdr)->te1[0].gfimr;
	fisr = fisr & fmask;

	for (i = 0; i < MAX_T1_CHANNELS; i++) {
		if(!((sptr = &T1stuff[id][i])->init))
			continue;
		if(fisr & (1 << i))
			check_again |= process_t1(sptr);
	}

	return(check_again);
}

/* -------------------------------------------------------------------------- */
/* This routine get called from the global function to hook up the isr 
 * If the T1 the first to require ISR services, this will be called 
 * if the ISR was hooked up by another technology, this will not get called
 */
static void Glob2T1hookISR(STUFF *sptr, void *device_isr, int set_clear)
{

	if(set_clear) {
		/* Hook up the interrupt */
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_HOOKINTERRUPT, device_isr, 
			USER_DEFINED_HOOKINT_VALUE);
	}
	else {
		/* Unhook the interrupt */
		(*sptr->cback)(sptr->lnPtr, TE1DCLBK_UNHOOKINTERRUPT);
	}
}

/*------------------------------------------------------------------------*/
/* Routine to control hooking up and unhooking interrupts
 */
static int hook_isr(STUFF *sptr, int set_clear, int flag)
{
int retval = 1;
int previous = sptr->int_flag;

	if(set_clear) {
		/* check to see if need to hook interrupt */

		sptr->int_flag |= flag;

		switch(flag) {
		case INTFLAG_UTIL:
		case INTFLAG_TR008:
		default:
			break;
		case INTFLAG_ONESEC:
		case INTFLAG_ALM:
		case INTFLAG_T1403:
		case INTFLAG_LOOP:
			/* Hook the interrupt here */

			if(!sptr->isr_hooked) {
			   sptr->isr_hooked = 1;
			   NCID_GHOOKISR(sptr->d, DRVR_IDX_T1,
				set_clear, Glob2T1hookISR, sptr, T1_handler);
			}
			break;
		}
	}
	else {
		/* check to see if need to unhook interrupt */

		previous &= ~flag;

		switch(flag) {
		case INTFLAG_UTIL:
		case INTFLAG_TR008:
		default:
			break;
		case INTFLAG_ONESEC:
		case INTFLAG_ALM:
		case INTFLAG_T1403:
		case INTFLAG_LOOP:
			/* UnHook the interrupt here */

			if(
				(!(previous & INTFLAG_UTIL)) && 
				(!(previous & INTFLAG_ONESEC)) && 
				(!(previous & INTFLAG_ALM)) && 
				(!(previous & INTFLAG_T1403)) && 
				(!(previous & INTFLAG_LOOP)) && 
				(!(previous & INTFLAG_TR008)) &&
				sptr->isr_hooked
			) {
			   NCID_GHOOKISR(sptr->d, DRVR_IDX_T1,
				set_clear, Glob2T1hookISR, sptr, T1_handler);
			   sptr->isr_hooked = 0;
		
			}
			break;
		}
		sptr->int_flag &= ~flag;
	}

	return(retval);
}

/*------------------------------------------------------------------------*/
/* Validate the line before being bound to a device
 */
static int _te1_valid_channel(GDEVICE *d, int subchannel)
{
	/* Single channel device no validation needed */
	return(1);
}


/* -------------------------------------------------------------------------- */
/* 
 * The include that follows below incorporate the standard T1
 * driver construct.  This Construct is common accross all T1
 * drivers.  You do not want to change the contents of the file that
 * is included below since it will change ALL T1 drivers.
 */
/* -------------------------------------------------------------------------- */

#include "apiTE1.inc"
