/* 
 * Copyright 1997-2005 NComm, Inc.  All rights reserved.
 * 
 *
 *                     *** Important Notice ***
 * The software contained in this file may only be used within a 
 * valid license agreement between your company and NComm, Inc. 
 * The license agreement includes the definition of a PROJECT which 
 * defines the scope that this software can be used in.  Any use outside of
 * the definition of the PROJECT is prohibited without executing an additional 
 * agreement with NComm, Inc. Please refer to you license agreement for 
 * the definition of the PROJECT.
 *
 * Verification of your company's license agreement and copies of 
 * that agreement also may be obtained from:
 *
 * NComm, Inc.
 * 254 N Broadway
 * Suite 106
 * Salem, NH 03079
 * (603) 893-6186
 * sales@ncomm.com
 *
 */


#ifndef NCITE1_H
#define NCITE1_H



/*
 * These are used at all levels of the NCI T1/E1/LIU packages, including
 * the drivers.
 */



/*--------------------------------------------------------------------------*/
typedef enum {
	NCIT1 = 100,
	NCIE1
} TE1_TYPE;			/* define the T1/E1 format */



typedef enum {
	FF_OTHER=1,
	FF_ESF=2,
	FF_D4=3,
	FF_E1=4,
	FF_E1_CRC=5,
	FF_E1_MF=6,
	FF_E1_CRC_MF=7,
	FF_SLC96=8
} TE1_FORMAT;			/* define the frame format */



typedef enum {
	LC_JBZS=1,
	LC_B8ZS=2,
	LC_HDB3=3,
	LC_ZBTSI=4,
	LC_AMI=5,
	LC_OTHER=6
} TE1_CODING;			/* define the line coding */



typedef struct {
	TE1_TYPE	t1e1;
	TE1_FORMAT	framing;
	TE1_CODING	coding;
	int		lbo;		/* raw signed value */
	int		wStart;	/* internal TMS use only.  App can ignore */
} TE1_CONFIG;



typedef struct _ds0Config {
	void *model;
	void *submodel;
	int idlepat;
} TE1_DS0CONFIG;



/* The package goes to the driver to get this info once per second when the
 * T1-MMM is installed and the Performance Monitoring is enabled
 */
typedef struct _perfinfoT1 {
	unsigned long FEcount;		/* framing error count */
	unsigned long CVcount;		/* coding violations + EXzeros */
	unsigned long CRCcount;		/* crc error count */
	unsigned long EBcount;		/* errored block count */
	unsigned  int SLevent;		/* Rx slip event SET or CLEAR */
	unsigned  int SEFEevent;	/* severely errored framing event
					 * SET or CLEAR */
} T1_PERFINFO;



/* The package goes to the driver to get this info once per second when the
 * E1-MMM is installed and the Performance Monitoring is enabled
 */
typedef struct _perfinfoE1 {
	unsigned long CVcount;
	unsigned long CRCcount;
	unsigned long EBITcount;
	unsigned long FAScount;
} E1_PERFINFO;



/* E1 carries an array of five 8-bit values, one for each of the
 * five SaBit positions Sa4 through Sa8.  These enums are used as
 * array indexes.
 */
typedef enum {
	E1SABYTE4 = 0,
	E1SABYTE5,
	E1SABYTE6,
	E1SABYTE7,
	E1SABYTE8,
	E1SABYTE_MAX
} E1SABYTE_TYPE;


#endif	/* NCITE1_H */
